@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -34,59 +34,114 @@ A library finds tracks in a specified directory and adds them to an associated
 collection.
 """
 
-import os
-import time
-import os.path
-import shutil
-import logging
+from __future__ import with_statement
 from collections import deque
-
+import glib
 import gobject
 import gio
+import logging
+import os
+import os.path
+import shutil
+import threading
+import time
 
 from xl.nls import gettext as _
-from xl import common, xdg, event, metadata, settings, trax
-from xl.settings import SettingsManager
-
+from xl import (
+    common,
+    event,
+    metadata,
+    settings,
+    trax,
+    xdg
+)
 
 logger = logging.getLogger(__name__)
 
 COLLECTIONS = set()
 
-
 def get_collection_by_loc(loc):
     """
         gets the collection by a location.
 
         :param loc: Location of the collection
         :return: collection at location or None
-        :rtype: Collection
+        :rtype: :class:`Collection`
     """
     for c in COLLECTIONS:
         if c.loc_is_member(loc):
             return c
     return None
 
+class CollectionScanThread(common.ProgressThread):
+    """
+        Scans the collection
+    """
+    def __init__(self, collection, startup_scan=False, force_update=False):
+        """
+            Initializes the thread
+
+            :param collection: the collection to scan
+            :param startup_scan: Only scan libraries scanned at startup
+            :param force_update: Update files regardless whether they've changed
+        """
+        common.ProgressThread.__init__(self)
+        
+        self.startup_scan = startup_scan
+        self.force_update = force_update
+        self.collection = collection
+
+    def stop(self):
+        """
+            Stops the thread
+        """
+        self.collection.stop_scan()
+        common.ProgressThread.stop(self)
+
+    def run(self):
+        """
+            Runs the thread
+        """
+        event.add_callback(self.on_scan_progress_update,
+            'scan_progress_update')
+
+        self.collection.rescan_libraries(startup_only=self.startup_scan,
+                                         force_update=self.force_update)
+
+        event.remove_callback(self.on_scan_progress_update,
+            'scan_progress_update')
+
+    def on_scan_progress_update(self, type, collection, progress):
+        """
+            Notifies about progress changes
+        """
+        if progress < 100:
+            self.emit('progress-update', progress)
+        else:
+            self.emit('done')
+
 class Collection(trax.TrackDB):
     """
         Manages a persistent track database.
 
-        :param args: see :class:`xl.trackdb.TrackDB`
+        :param args: see :class:`xl.trax.trackdb.TrackDB`
 
         Simple usage:
 
         >>> from xl.collection import *
-        >>> c = Collection("Test Collection")
-        >>> c.add_library(Library("./tests/data"))
-        >>> c.rescan_libraries()
-        >>> tracks = list(c.search('artist="TestArtist"'))
+        >>> from xl.trax import search
+        >>> collection = Collection("Test Collection")
+        >>> collection.add_library(Library("./tests/data"))
+        >>> collection.rescan_libraries()
+        >>> tracks = [i.track for i in search.search_tracks_from_string(
+        ...     collection, ('artist==TestArtist'))]
         >>> print len(tracks)
         5
         >>>
     """
     def __init__(self, name, location=None, pickle_attrs=[]):
         global COLLECTIONS
-        self.libraries = dict()
+        self.libraries = {}
         self._scanning = False
         self._scan_stopped = False
         self._running_count = 0
@@ -186,7 +241,7 @@ class Collection(trax.TrackDB):
         """
         return self.libraries.values()
 
-    def rescan_libraries(self):
+    def rescan_libraries(self, startup_only=False, force_update=False):
         """
             Rescans all libraries associated with this Collection
         """
@@ -205,10 +260,14 @@ class Collection(trax.TrackDB):
 
         scan_interval = 20
 
-        for library in self.libraries.values():
+        for library in self.libraries.itervalues():
+            
+            if not force_update and startup_only and not (library.monitored and library.startup_scan):
+                continue
+            
             event.add_callback(self._progress_update, 'tracks_scanned',
                 library)
-            library.rescan(notify_interval=scan_interval)
+            library.rescan(notify_interval=scan_interval, force_update=force_update)
             event.remove_callback(self._progress_update, 'tracks_scanned',
                 library)
             self._running_total_count += self._running_count
@@ -266,8 +325,10 @@ class Collection(trax.TrackDB):
         for k, v in self.libraries.iteritems():
             l = {}
             l['location'] = v.location
-            l['realtime'] = v.realtime
+            l['monitored'] = v.monitored
+            l['realtime'] = v.monitored
             l['scan_interval'] = v.scan_interval
+            l['startup_scan'] = v.startup_scan
             _serial_libraries.append(l)
         return _serial_libraries
 
@@ -279,7 +340,8 @@ class Collection(trax.TrackDB):
         """
         for l in _serial_libraries:
             self.add_library( Library( l['location'],
-                        l['realtime'], l['scan_interval'] ))
+                    l.get('monitored', l.get('realtime')),
+                    l['scan_interval'], l.get('startup_scan', True) ))
 
     _serial_libraries = property(serialize_libraries, unserialize_libraries)
 
@@ -296,16 +358,165 @@ class Collection(trax.TrackDB):
             for prefix, lib in self.libraries.iteritems():
                 lib.delete(tr.get_loc_for_io())
 
+class LibraryMonitor(gobject.GObject):
+    """
+        Monitors library locations for changes
+    """
+    __gproperties__ = {
+        'monitored': (
+            gobject.TYPE_BOOLEAN,
+            'monitoring state',
+            'Whether to monitor this library',
+            False,
+            gobject.PARAM_READWRITE
+        )
+    }
+    __gsignals__ = {
+        'location-added': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            [gio.File]
+        ),
+        'location-removed': (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            [gio.File]
+        )
+    }
+    
+    def __init__(self, library):
+        """
+            :param library: the library to monitor
+            :type library: :class:`Library`
+        """
+        gobject.GObject.__init__(self)
+
+        self.__library = library
+        self.__root = gio.File(library.location)
+        self.__monitored = False
+        self.__monitors = {}
+        self.__queue = {}
+        self.__lock = threading.RLock()
+
+    def do_get_property(self, property):
+        """
+            Gets GObject properties
+        """
+        if property.name == 'monitored':
+            return self.__monitored
+        else:
+            raise AttributeError('unknown property %s' % property.name)
+
+    def do_set_property(self, property, value):
+        """
+            Sets GObject properties
+        """
+        if property.name == 'monitored':
+            if value != self.__monitored:
+                self.__monitored = value
+                update_thread = threading.Thread(target=self.__update_monitors)
+                update_thread.daemon = True
+                glib.idle_add(update_thread.start)
+        else:
+            raise AttributeError('unkown property %s' % property.name)
+
+    def __update_monitors(self):
+        """
+            Sets up or removes library monitors
+        """
+        with self.__lock:
+            if self.props.monitored:
+                logger.debug('Setting up library monitors')
+
+                for directory in common.walk_directories(self.__root):
+                    monitor = directory.monitor_directory()
+                    monitor.connect('changed', self.on_location_changed)
+                    self.__monitors[directory] = monitor
+
+                    self.emit('location-added', directory)
+            else:
+                logger.debug('Removing library monitors')
+
+                for directory, monitor in self.__monitors.iteritems():
+                    monitor.cancel()
+
+                    self.emit('location-removed', directory)
+
+                self.__monitors = {}
+                
+    def __process_change_queue(self, gfile):
+        if gfile in self.__queue:
+            added_tracks = trax.util.get_tracks_from_uri(gfile.get_uri())
+            for tr in added_tracks:
+                tr.read_tags()
+            self.__library.collection.add_tracks(added_tracks)
+            del self.__queue[gfile]
+
+    def on_location_changed(self, monitor, gfile, other_gfile, event):
+        """
+            Updates the library on changes of the location
+        """
+        
+        if event == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+            self.__process_change_queue(gfile)
+        elif event == gio.FILE_MONITOR_EVENT_CREATED or \
+             event == gio.FILE_MONITOR_EVENT_CHANGED:
+            
+            # Enqueue tracks retrieval
+            if gfile not in self.__queue:
+                self.__queue[gfile] = True
+            
+                # File monitor only emits the DONE_HINT when using inotify,
+                # and only on single files. Give it some time, but don't 
+                # lose the change notification
+                glib.timeout_add(500, self.__process_change_queue, gfile)
+            
+            # Set up new monitor if directory
+            fileinfo = gfile.query_info('standard::type')
+
+            if fileinfo.get_file_type() == gio.FILE_TYPE_DIRECTORY and \
+               gfile not in self.__monitors:
+                
+                for directory in common.walk_directories(gfile):
+                    monitor = directory.monitor_directory()
+                    monitor.connect('changed', self.on_location_changed)
+                    self.__monitors[directory] = monitor
+
+                    self.emit('location-added', directory)
+
+        elif event == gio.FILE_MONITOR_EVENT_DELETED:
+            removed_tracks = []
+
+            track = trax.Track(gfile.get_uri())
+
+            if track in self.__library.collection:
+                # Deleted file was a regular track
+                removed_tracks += [track]
+            else:
+                # Deleted file was most likely a directory
+                for track in self.__library.collection:
+                    track_gfile = gio.File(track.get_loc_for_io())
+
+                    if track_gfile.has_prefix(gfile):
+                        removed_tracks += [track]
+
+            self.__library.collection.remove_tracks(removed_tracks)
+
+            # Remove obsolete monitors
+            removed_directories = [d for d in self.__monitors \
+                if d == gfile or d.has_prefix(gfile)]
+
+            for directory in removed_directories:
+                self.__monitors[directory].cancel()
+                del self.__monitors[directory]
+                
+                self.emit('location-removed', directory)
+
 class Library(object):
     """
         Scans and watches a folder for tracks, and adds them to
         a Collection.
 
-        :param location: the directory this library will scan
-        :type location: string
-        :param collection: the Collection to associate with
-        :type collection: :class:`Collection`
-
         Simple usage:
 
         >>> from xl.collection import *
@@ -320,15 +531,25 @@ class Library(object):
         5
         >>>
     """
-    def __init__(self, location, realtime=False, scan_interval=0):
+    def __init__(self, location, monitored=False, scan_interval=0, startup_scan=False):
         """
             Sets up the Library
+
+            :param location: the directory this library will scan
+            :type location: string
+            :param monitored: whether the library should update its
+                collection at changes within the library's path
+            :type monitored: bool
+            :param scan_interval: the interval for automatic rescanning
+            :type scan_interval: int
         """
         self.location = location
         self.scan_interval = scan_interval
         self.scan_id = 0
         self.scanning = False
-        self.realtime = False
+        self._startup_scan = startup_scan
+        self.monitor = LibraryMonitor(self)
+        self.monitor.props.monitored = monitored
 
         self.collection = None
         self.set_rescan_interval(scan_interval)
@@ -355,49 +576,67 @@ class Library(object):
 
         self.collection = collection
 
-    def _scan_locations(self, locations):
+    def get_monitored(self):
         """
-            Scans locations for tracks and adds them to the collection
-
-            :param locations: a list of locations to check
+            Whether the library should be monitored for changes
         """
-        db = self.collection
-        for fullpath in locations:
-            tr = db.get_track_by_loc(fullpath)
-            if tr:
-                # check to see if we need to scan this track
-                mtime = os.path.getmtime(fullpath)
-                if unicode(mtime) == tr['__modified']:
-                    continue
-                else:
-                    tr.read_tags()
-                    continue
+        return self.monitor.props.monitored
 
-            tr = trax.Track(fullpath)
-            if tr._scan_valid:
-                db.add(tr)
-
-    def _remove_locations(self, locations):
+    def set_monitored(self, monitored):
         """
-            Removes tracks at the specified locations from the collection
-
-            :param locations: the paths to remove
+            Enables or disables monitoring of the library
+            
+            :param monitored: Whether to monitor the library
+            :type monitored: bool
         """
-        for loc in locations:
-            try:
-                track = self.collection.tracks[loc]
-            except KeyError:
-                common.log_exception(log=logger)
-                continue
+        self.monitor.props.monitored = monitored
+        self.collection.serialize_libraries()
+        self.collection._dirty = True
 
-            self.collection.remove(track)
+    monitored = property(get_monitored, set_monitored)
+
+    def get_rescan_interval(self):
+        """
+            :return: the scan interval in seconds
+        """
+        return self.scan_interval
+
+    def set_rescan_interval(self, interval):
+        """
+            Sets the scan interval in seconds.  If the interval is 0 seconds,
+            the scan interval is stopped
+
+            :param interval: scan interval in seconds
+            :type interval: int
+        """
+        if not interval:
+            if self.scan_id:
+                glib.source_remove(self.scan_id)
+                self.scan_id = 0
+        else:
+            if self.scan_id:
+                glib.source_remove(self.scan_id)
+
+            self.scan_id = glib.timeout_add_seconds(interval, self.rescan)
+
+        self.scan_interval = interval
+        
+    def get_startup_scan(self):
+        return self._startup_scan
+    
+    def set_startup_scan(self, value):
+        self._startup_scan = value
+        self.collection.serialize_libraries()
+        self.collection._dirty = True
+    
+    startup_scan = property(get_startup_scan, set_startup_scan)
 
     def _count_files(self):
         """
             Counts the number of files present in this directory
         """
         count = 0
-        for file in self._walk(gio.File(self.location)):
+        for file in common.walk(gio.File(self.location)):
             if self.collection:
                 if self.collection._scan_stopped:
                     break
@@ -424,20 +663,20 @@ class Library(object):
             return
 
         def joiner(value):
-            if not value or type(value) in (str, unicode):
+            if not value or isinstance(value, basestring):
                 return value
             else:
                 try:
                     return u"\u0000".join(value)
-                except TypeError:
-                    return value
+                except UnicodeDecodeError:
+                    return "\0".join(value)
 
         try:
             basedir = joiner(tr.get_tag_raw('__basedir'))
             album = joiner(tr.get_tag_raw('album'))
             artist = joiner(tr.get_tag_raw('artist'))
-        except UnicodeDecodeError: #TODO: figure out why this happens
-            logger.warning("Encoding error, skipping compilation check")
+        except Exception:
+            logger.warning("Error while checking for compilation: " + `tr`)
             return
         if not basedir or not album or not artist: return
         album = album.lower()
@@ -452,62 +691,23 @@ class Library(object):
             common.log_exception(log=logger)
             return
 
-        if ccheck[basedir][album] and not \
-            artist in ccheck[basedir][album]:
+        if ccheck[basedir][album] and \
+                artist not in ccheck[basedir][album]:
             if not (basedir, album) in compilations:
                 compilations.append((basedir, album))
-                logger.info("Compilation %(album)s detected in %(dir)s" %
-                        {'album':album, 'dir':basedir})
+                logger.debug("Compilation %(album)r detected in %(dir)r" %
+                        {'album': album, 'dir': basedir})
 
         ccheck[basedir][album].append(artist)
 
-    def _walk(self, root):
-        """
-            Walk through a gio directory, returning each file in turn.
-
-            Files are enumerated in the following order: first the directory,
-            then the files in that directory. Once one directory's files have
-            all been listed, it moves on to the next directory. Order of files
-            within a dir and order of dir traversal is not specified.
-
-            :root: A gio.File representing the directory to walk through
-
-            returns a generator object that yields each gio.File in turn
-        """
-        queue = deque()
-        queue.append(root)
-
-        while len(queue) > 0:
-            dir = queue.pop()
-            yield dir
-            try:
-                for fileinfo in dir.enumerate_children("standard::type,"
-                        "standard::is-symlink,standard::name,"
-                        "standard::symlink-target,time::modified"):
-                    fil = dir.get_child(fileinfo.get_name())
-                    # FIXME: recursive symlinks could cause an infinite loop
-                    if fileinfo.get_is_symlink():
-                        target = fileinfo.get_symlink_target()
-                        if not "://" in target and not os.path.isabs(target):
-                            fil2 = dir.get_child(target)
-                        else:
-                            fil2 = gio.File(target)
-                        # already in the collection, we'll get it anyway
-                        if fil2.has_prefix(root):
-                            continue
-                    type = fileinfo.get_file_type()
-                    if type == gio.FILE_TYPE_DIRECTORY:
-                        queue.append(fil)
-                    elif type == gio.FILE_TYPE_REGULAR:
-                        yield fil
-            except gio.Error: # why doesnt gio offer more-specific errors?
-                pass
-
-    def update_track(self, gloc):
+    def update_track(self, gloc, force_update=False):
         """
             Rescan the track at a given location
 
-            :gloc: A gio.File representing the location
+            :param gloc: the location
+            :type gloc: :class:`gio.File`
+            :param force_update: Force update of file (default only updates file
+                                 when mtime has changed)
 
             returns: the Track object, None if it could not be updated
         """
@@ -517,7 +717,7 @@ class Library(object):
         mtime = gloc.query_info("time::modified").get_modification_time()
         tr = self.collection.get_track_by_loc(uri)
         if tr:
-            if tr.get_tag_raw('__modified') < mtime:
+            if force_update or tr.get_tag_raw('__modified') < mtime:
                 tr.read_tags()
                 tr.set_tag_raw('__modified', mtime)
         else:
@@ -533,7 +733,7 @@ class Library(object):
                 self.collection.add(tr)
         return tr
 
-    def rescan(self, notify_interval=None):
+    def rescan(self, notify_interval=None, force_update=False):
         """
             Rescan the associated folder and add the contained files
             to the Collection
@@ -546,7 +746,7 @@ class Library(object):
         if self.scanning:
             return
 
-        logger.info("Scanning library: %s" % self.location)
+        logger.info("Scanning library: %s", self.location)
         self.scanning = True
         db = self.collection
         libloc = gio.File(self.location)
@@ -555,7 +755,7 @@ class Library(object):
         dirtracks = deque()
         compilations = deque()
         ccheck = {}
-        for fil in self._walk(libloc):
+        for fil in common.walk(libloc):
             count += 1
             type = fil.query_info("standard::type").get_file_type()
             if type == gio.FILE_TYPE_DIRECTORY:
@@ -577,7 +777,7 @@ class Library(object):
                 compilations = deque()
                 ccheck = {}
             elif type == gio.FILE_TYPE_REGULAR:
-                tr = self.update_track(fil)
+                tr = self.update_track(fil, force_update=force_update)
                 if not tr:
                     continue
 
@@ -597,6 +797,7 @@ class Library(object):
 
             if self.collection and self.collection._scan_stopped:
                 self.scanning = False
+                logger.info("Scan canceled")
                 return
 
             # progress update
@@ -610,7 +811,7 @@ class Library(object):
 
 
         removals = deque()
-        for k, tr in self.collection.tracks.iteritems():
+        for tr in self.collection.tracks.itervalues():
             tr = tr._track
             loc = tr.get_loc_for_io()
             if not loc:
@@ -629,49 +830,10 @@ class Library(object):
         for tr in removals:
             logger.debug(u"Removing %s"%unicode(tr))
             self.collection.remove(tr)
+            
+        logger.info("Scan completed: %s", self.location)
         self.scanning = False
 
-    def is_realtime(self):
-        """
-            :return: True if this is library is being watched
-        """
-        return self.realtime
-
-    def set_realtime(self, realtime):
-        """
-            Set to True if you want this library to be monitored
-
-            :raises PyInotifyNotSupportedException: The collection is not set
-                to realtime or :mod:`pyinotify` is not available
-        """
-        pass # TODO: reimplement with gio.FileMonitor
-
-    def get_rescan_interval(self):
-        """
-            :return: the scan interval in seconds
-        """
-        return self.scan_interval
-
-    def set_rescan_interval(self, interval):
-        """
-            Sets the scan interval in seconds.  If the interval is 0 seconds,
-            the scan interval is stopped
-
-            :param interval: scan interval in seconds
-            :type interval: int
-        """
-        if not interval:
-            if self.scan_id:
-                gobject.source_remove(self.scan_id)
-                self.scan_id = 0
-        else:
-            if self.scan_id:
-                gobject.source_remove(self.scan_id)
-
-            self.scan_id = gobject.timeout_add_seconds(interval, self.rescan)
-
-        self.scan_interval = interval
-
     def add(self, loc, move=False):
         """
             Copies (or moves) a file into the library and adds it to the
