@@ -31,24 +31,33 @@
 #
 # Also takes care of parsing commandline options.
 
+import errno
 import logging
 import logging.handlers
 import os
 import platform
 import sys
+import threading
 
-try:
-    import gio
-except ImportError:
-    # on Win32 using the GStreamer SDK, requires import of
-    # pygtk first
-    import pygtk
-    pygtk.require('2.0')
-    import gio
-
-from xl import common, xdg
 from xl.nls import gettext as _
 
+# Imported later to avoid PyGTK imports just for --help.
+gio = common = xdg = None
+
+def _do_heavy_imports():
+    global gio, common, xdg
+
+    try:
+        import gio
+    except ImportError:
+        # on Win32 using the GStreamer SDK, requires import of
+        # pygtk first
+        import pygtk
+        pygtk.require('2.0')
+        import gio
+
+    from xl import common, xdg
+
 # placeholder, - xl.version can be slow to import, which would slow down
 # cli args. Thus we import __version__ later.
 __version__ = None
@@ -80,12 +89,19 @@ class Exaile(object):
         """
         self.quitting = False
         self.loading = True
-        (self.options, self.args) = self.get_options().parse_args()
+
+        # NOTE: This automatically exits on --help.
+        try:
+            (self.options, self.args) = self.get_options().parse_args()
+        except UnicodeDecodeError:
+            (self.options, self.args) = self.get_options(unicode_bug_happened=True).parse_args()
 
         if self.options.ShowVersion:
             self.version()
             return
 
+        _do_heavy_imports()
+
         if self.options.UseDataDir:
             xdg.data_dirs.insert(1, self.options.UseDataDir)
 
@@ -98,13 +114,23 @@ class Exaile(object):
             xdg.config_dirs.insert(0, xdg.config_home)
             xdg.cache_home = self.options.UseAllDataDir
             
-        xdg._make_missing_dirs()
+        try:
+            xdg._make_missing_dirs()
+        except OSError, e:
+            print >> sys.stderr, 'ERROR: Could not create configuration directories: %s' % str(e)
+            return
+            
 
         # Make event debug imply debug
         if self.options.DebugEvent:
             self.options.Debug = True
 
-        self.setup_logging()
+        try:
+            self.setup_logging()
+        except OSError, e:
+            print >> sys.stderr, 'ERROR: could not setup logging: %s' % str(e)
+            return
+        
         global logger
         logger = logging.getLogger(__name__)
 
@@ -171,9 +197,20 @@ class Exaile(object):
             from xl import settings
         except common.VersionError:
             common.log_exception(log=logger)
-            exit(1)
+            sys.exit(1)
             
         logger.debug("Settings loaded from %s" % settings.location)
+        
+        # display locale information if available
+        try:
+            import locale
+            lc, enc = locale.getlocale()
+            if enc is not None:
+                logger.info("Using %s %s locale" % (lc, enc))
+            else:
+                logger.info("Using unknown locale")
+        except:
+            pass
 
         splash = None
 
@@ -210,12 +247,13 @@ class Exaile(object):
 
         # Initialize plugin manager
         from xl import plugins
+        self.plugins = plugins.PluginsManager(self)
+        
         if not self.options.SafeMode:
             logger.info("Loading plugins...")
-            self.plugins = plugins.PluginsManager(self)
+            self.plugins.load_enabled()
         else:
             logger.info("Safe mode enabled, not loading plugins.")
-            self.plugins = plugins.PluginsManager(self, load=False)
 
         # Initialize the collection
         logger.info("Loading collection...")
@@ -225,7 +263,7 @@ class Exaile(object):
                     location=os.path.join(xdg.get_data_dir(), 'music.db'))
         except common.VersionError:
             common.log_exception(log=logger)
-            exit(1)
+            sys.exit(1)
 
         from xl import event
         # Set up the player and playback queue
@@ -251,11 +289,26 @@ class Exaile(object):
         self.devices = devices.DeviceManager()
         event.log_event("device_manager_ready", self, None)
 
-        # Initialize HAL
+        # Initialize dynamic device discovery interface
+        # -> if initialized and connected, then the object is not None
+        
+        self.udisks2 = None
+        self.udisks = None
+        self.hal = None
+        
         if self.options.Hal:
             from xl import hal
-            self.hal = hal.HAL(self.devices)
-            self.hal.connect()
+                
+            udisks2 = hal.UDisks2(self.devices)
+            if udisks2.connect():
+                self.udisks2 = udisks2
+            else:
+                udisks = hal.UDisks(self.devices)
+                if udisks.connect():
+                    self.udisks = udisks
+                else:
+                    self.hal = hal.HAL(self.devices)
+                    self.hal.connect()
         else:
             self.hal = None
 
@@ -289,6 +342,10 @@ class Exaile(object):
                 self.gui.open_uri(args[0], play=True)
             for arg in args[1:]:
                 self.gui.open_uri(arg)
+            
+            # kick off autoscan of libraries
+            # -> don't do it in command line mode, since that isn't expected
+            self.gui.rescan_collection_with_progress(True)
 
         if restore:
             player.QUEUE._restore_player_state(
@@ -400,19 +457,39 @@ class Exaile(object):
         logfile.setFormatter(formatter)
         logging.getLogger("").addHandler(logfile)
 
-    def get_options(self):
+    def get_options(self, unicode_bug_happened=False):
         """
             Get the options for exaile
         """
         from optparse import OptionParser, OptionGroup, IndentedHelpFormatter
 
-        class OverrideHelpFormatter(IndentedHelpFormatter):
-            """
-                Merely for translation purposes
-            """
-            def format_usage(self, usage):
-                return '%s\n' % usage
-
+        if unicode_bug_happened:
+            
+            #
+            # Bug: https://bugs.launchpad.net/exaile/+bug/1154420
+            #
+            # For some locales, python doesn't merge the options and
+            # the headings and our translated text correctly. Unfortunately,
+            # there doesn't seem to be a good way to deal with the problem
+            # on Python 2.x . If we disable the usage/heading, at least
+            # the options will display, despite filling all the text as ???. 
+            #
+            
+            print >> sys.stderr, "exaile: Warning: Unicode error displaying --help, check locale settings"
+            
+            class OverrideHelpFormatter(IndentedHelpFormatter):
+                def format_usage(self, usage):
+                    return ''
+                def format_heading(self, heading):
+                    return ''
+        else:
+            class OverrideHelpFormatter(IndentedHelpFormatter):
+                """
+                    Merely for translation purposes
+                """
+                def format_usage(self, usage):
+                    return '%s\n' % usage
+        
         usage = _("Usage: exaile [OPTION]... [URI]")
         optionlabel = _('Options') # Merely for translation purposes
         p = OptionParser(usage=usage, add_help_option=False,
@@ -569,7 +646,7 @@ class Exaile(object):
     def version(self):
         from xl.version import __version__
         print "Exaile", __version__
-        exit()
+        sys.exit(0)
 
     def _add_default_playlists(self):
         """
@@ -611,9 +688,10 @@ class Exaile(object):
             import glib
             loop = glib.MainLoop()
             context = loop.get_context()
-            self.__mainloop(context)
+            t = threading.Thread(target=self.__mainloop, args=(context,))
+            t.daemon = True
+            t.start()
 
-    @common.threaded
     def __mainloop(self, context):
         while 1:
             try:
@@ -628,6 +706,48 @@ class Exaile(object):
             Returns the current version
         """
         return __version__
+    
+    def get_user_agent_string(self, plugin_name=None):
+        '''
+            Returns an approrpiately formatted User-agent string for 
+            web requests. When possible, plugins should use this to
+            format user agent strings.
+            
+            Users can control this agent string by manually setting
+            general/user_agent and general/user_agent_w_plugin in settings.ini
+            
+            :param plugin_name: the name of the plugin
+        '''
+        
+        version = __version__
+        if '+' in version:  # strip out revision identifier
+            version = version[:version.index('+')]
+        
+        fmt = {
+            'version': version
+        }
+        
+        if not hasattr(self, '_user_agent_no_plugin'):
+                    
+            from xl import settings
+            
+            default_no_plugin = 'Exaile/%(version)s (+http://www.exaile.org)'
+            default_plugin = 'Exaile/%(version)s %(plugin_name)s/%(plugin_version)s (+http://www.exaile.org)'
+        
+            self._user_agent_no_plugin = \
+                settings.get_option('general/user_agent', default_no_plugin)
+            self._user_agent_w_plugin = \
+                settings.get_option('general/user_agent_w_plugin', default_plugin)
+        
+        if plugin_name is not None:
+            plugin_info = self.plugins.get_plugin_info(plugin_name)
+            
+            fmt['plugin_name'] = plugin_info['Name'].replace(' ', '')
+            fmt['plugin_version'] = plugin_info['Version']
+            
+            return self._user_agent_w_plugin % fmt
+        else:
+            return self._user_agent_no_plugin % fmt
 
     def quit(self, restart=False):
         """
@@ -690,7 +810,7 @@ class Exaile(object):
 
         logger.info("Bye!")
         logging.shutdown()
-        exit()
+        sys.exit(0)
 
 def exaile():
     if not Exaile._exaile:
