@@ -29,35 +29,57 @@
     code in a gettext fashion without a hard depend on gettext itself.
 """
 
+import sys
 import locale
-
-from xl import xdg
+import os.path
 
 try:
     # Set to user default, gracefully fallback on C otherwise
     locale.setlocale(locale.LC_ALL, '')
-except locale.Error:
-    pass
+except locale.Error, e:
+    # Error message copied from bzr
+    sys.stderr.write('exaile: Warning: %s\n'
+                     '  Exaile could not set the application locale, this\n'
+                     '  may cause language-specific problems. To investigate this\n'
+                     '  issue, look at the output of the locale tool.\n' % e)
 
 try:
     import gettext as gettextmod
 
-    # Required for gtk.Builder messages
-    try:
-        locale.textdomain('exaile')
-    except AttributeError: # E.g. Windows
-        pass
-    # Required for dynamically added messages
-    gettextmod.textdomain('exaile')
+    def __setup_locale():
 
-    if xdg.local_hack: # running from source dir, so we have to set the paths
-        import os.path
-        locale_path = os.path.join(xdg.exaile_dir, 'po')
+        # Required for gtk.Builder messages
         try:
-            locale.bindtextdomain('exaile', locale_path)
+            locale.textdomain('exaile')
         except AttributeError: # E.g. Windows
             pass
-        gettextmod.bindtextdomain('exaile', locale_path)
+        
+        # Required for dynamically added messages
+        gettextmod.textdomain('exaile')
+    
+        locale_path = None
+        exaile_path = os.environ['EXAILE_DIR']
+    
+        # If running from source dir, we have to set the paths.
+        # (The test is equivalent to xdg.local_hack but without the xdg import,
+        # which pulls in glib.)
+        if os.path.exists(os.path.join(exaile_path, 'po')):
+            locale_path = os.path.join(exaile_path, 'po')
+        
+        # Detect the prefix, to see if we need to correct the locale path
+        elif exaile_path.endswith(os.path.join('lib', 'exaile')):
+            exaile_prefix = exaile_path[:-len(os.path.join('lib', 'exaile'))]
+            if os.path.exists(os.path.join(exaile_prefix, 'share', 'locale')):
+                locale_path = os.path.join(exaile_prefix, 'share', 'locale')
+        
+        if locale_path is not None:
+            try:
+                locale.bindtextdomain('exaile', locale_path)
+            except AttributeError: # E.g. Windows
+                pass
+            gettextmod.bindtextdomain('exaile', locale_path)
+
+    __setup_locale()
 
     gettextfunc = gettextmod.gettext
 
