@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -24,7 +24,12 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+    Allows for playback and queue control
+"""
+
 __all__ = [
+        'adapters',
         'pipe',
         'queue',
         'engine_normal',
@@ -33,25 +38,36 @@ __all__ = [
 
 import gobject
 gobject.threads_init()
-
-from xl.nls import gettext as _
-from xl import settings
 import logging
+import os
+
+from xl import settings, xdg
+
+import queue
+
 logger = logging.getLogger(__name__)
 
-def get_player():
+def get_player(*args, **kwargs):
     pname = settings.get_option("player/engine", "normal")
     if pname == "normal":
         logger.debug("Normal playback engine selected.")
         from xl.player.engine_normal import NormalPlayer
-        return NormalPlayer
+        return NormalPlayer(*args, **kwargs)
     elif pname == "unified":
         logger.debug("Unified playback engine selected.")
         from xl.player.engine_unified import UnifiedPlayer
-        return UnifiedPlayer
+        return UnifiedPlayer(*args, **kwargs)
+    elif pname == 'rtfd_hack': # allows building docs
+        return None
     else:
         logger.warning("Couldn't find specified playback engine, "
                 "falling back to normal.")
         from xl.player.engine_normal import NormalPlayer
-        return NormalPlayer
+        return NormalPlayer(*args, **kwargs)
+
+
+# TODO: write a better interface than this
+PLAYER = get_player('player')
+QUEUE = queue.PlayQueue(PLAYER, 'queue',
+        location=os.path.join(xdg.get_data_dir(), 'queue.state'))
 
