@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -34,12 +34,18 @@ import tarfile
 import traceback
 
 from xl.nls import gettext as _
-from xl import xdg, common, settings
+from xl import ( 
+    common, 
+    event,
+    settings,
+    xdg
+)
 
 logger = logging.getLogger(__name__)
 
 class InvalidPluginError(Exception):
-    pass
+    def __str__(self):
+        return str(self.args[0])
 
 class PluginsManager(object):
     def __init__(self, exaile, load=True):
@@ -48,11 +54,10 @@ class PluginsManager(object):
         if xdg.local_hack:
             self.plugindirs.insert(1, os.path.join(xdg.exaile_dir, 'plugins'))
 
-        for dir in self.plugindirs:
-            try:
-                os.makedirs(dir)
-            except:
-                pass
+        try:
+            os.makedirs(self.plugindirs[0])
+        except:
+            pass
 
         self.plugindirs = [ x for x in self.plugindirs if os.path.exists(x) ]
         self.loaded_plugins = {}
@@ -62,16 +67,13 @@ class PluginsManager(object):
 
         self.load = load
 
-        if self.load:
-            self.load_enabled()
-
     def __findplugin(self, pluginname):
         for dir in self.plugindirs:
             path = os.path.join(dir, pluginname)
             if os.path.exists(path):
                 return path
         return None
-
+    
     def load_plugin(self, pluginname, reload=False):
         if not reload and pluginname in self.loaded_plugins:
             return self.loaded_plugins[pluginname]
@@ -81,6 +83,8 @@ class PluginsManager(object):
             return False
         sys.path.insert(0, path)
         plugin = imp.load_source(pluginname, os.path.join(path,'__init__.py'))
+        if hasattr(plugin, 'plugin_class'):
+            plugin = plugin.plugin_class()
         sys.path = sys.path[1:]
         self.loaded_plugins[pluginname] = plugin
         return plugin
@@ -89,23 +93,44 @@ class PluginsManager(object):
         try:
             tar = tarfile.open(path, "r:*") #transparently supports gz, bz2
         except (tarfile.ReadError, OSError):
-            raise InvalidPluginError(_('Plugin archive is not in the correct '
-                'format'))
+            raise InvalidPluginError(
+                _('Plugin archive is not in the correct format.'))
 
         #ensure the paths in the archive are sane
         mems = tar.getmembers()
-        base = os.path.basename(path)[:-4].split("-")[0]
+        base = os.path.basename(path).split('.')[0]
         if os.path.isdir(os.path.join(self.plugindirs[0], base)):
-            raise InvalidPluginError(_('A plugin with the name "%s" is '
-                'already installed') % base)
+            raise InvalidPluginError(
+                _('A plugin with the name "%s" is already installed.') % base)
 
         for m in mems:
             if not m.name.startswith(base):
-                raise InvalidPluginError(_("Plugin archive contains an unsafe"
-                    " path"))
+                raise InvalidPluginError(
+                    _('Plugin archive contains an unsafe path.'))
 
         tar.extractall(self.plugindirs[0])
 
+    def __on_new_plugin_loaded(self, eventname, exaile, maybe_name, fn):
+        event.remove_callback(self.__on_new_plugin_loaded, eventname)
+        fn()
+
+    def __enable_new_plugin(self, plugin):
+        '''Sets up a new-style plugin. See helloworld plugin for details'''
+        
+        if hasattr(plugin, 'on_gui_loaded'):
+            if self.exaile.loading:
+                event.add_callback(self.__on_new_plugin_loaded, 'gui_loaded',
+                                   None, plugin.on_gui_loaded)
+            else:
+                plugin.on_gui_loaded()
+            
+        if hasattr(plugin, 'on_exaile_loaded'):
+            if self.exaile.loading:
+                event.add_callback(self.__on_new_plugin_loaded, 'exaile_loaded',
+                                   None, plugin.on_exaile_loaded)
+            else:
+                plugin.on_exaile_loaded()
+
     def uninstall_plugin(self, pluginname):
         self.disable_plugin(pluginname)
         for dir in self.plugindirs:
@@ -121,6 +146,8 @@ class PluginsManager(object):
             plugin = self.load_plugin(pluginname)
             if not plugin: raise Exception("Error loading plugin")
             plugin.enable(self.exaile)
+            if not inspect.ismodule(plugin):
+                self.__enable_new_plugin(plugin)
             self.enabled_plugins[pluginname] = plugin
             logger.debug("Loaded plugin %s" % pluginname)
             self.save_enabled()
@@ -134,6 +161,10 @@ class PluginsManager(object):
         try:
             plugin = self.enabled_plugins[pluginname]
             del self.enabled_plugins[pluginname]
+        except KeyError:
+            common.log_exception(logger, message="Plugin not found, possibly already disabled")
+            return False
+        try:
             plugin.disable(self.exaile)
             logger.debug("Unloaded plugin %s" % pluginname)
             self.save_enabled()
@@ -142,6 +173,7 @@ class PluginsManager(object):
             logger.warning("Unable to fully disable plugin %s" % pluginname)
             common.log_exception(logger)
             raise e
+        return True
 
     def list_installed_plugins(self):
         pluginlist = []
@@ -171,6 +203,43 @@ class PluginsManager(object):
             except ValueError:
                 pass # this happens on blank lines
         return infodict
+    
+    def is_compatible(self, info):
+        '''
+            Returns True if the plugin claims to be compatible with the
+            current platform.
+            
+            :param info: The data returned from get_plugin_info()
+        '''
+        platforms = info.get('Platforms', [])
+        if len(platforms) == 0:
+            platforms = [sys.platform]
+        
+        for platform in platforms:
+            if sys.platform.startswith(platform):
+                return True
+            
+        return False
+    
+    def is_potentially_broken(self, info):
+        '''
+            Returns True if one of the modules that the plugin requires is
+            not detected as available.
+        
+            :param info: The data returned from get_plugin_info()
+        '''
+        
+        modules = info.get('RequiredModules', [])
+        
+        for module in modules:
+            try:
+                mdata = imp.find_module(module)
+                if mdata[0] is not None:
+                    mdata[0].close()
+            except:
+                return True
+            
+        return False
 
     def get_plugin_default_preferences(self, pluginname):
         """
@@ -180,9 +249,9 @@ class PluginsManager(object):
         path = self.__findplugin(pluginname)
         plugin = imp.load_source(pluginname, os.path.join(path,'__init__.py'))
         try:
-            prefspane = plugin.get_prefs_pane()
-            for c in dir(prefspane):
-                attr = getattr(prefspane, c)
+            preferences_pane = plugin.get_preferences_pane()
+            for c in dir(preferences_pane):
+                attr = getattr(preferences_pane, c)
                 if inspect.isclass(attr):
                     try:
                         preflist[attr.name] = attr.default
