@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -55,7 +55,7 @@ class RadioManager(providers.ProviderHandler):
         """
         providers.register(self.servicename, station)
 
-    def on_new_provider(self, station):
+    def on_provider_added(self, station):
         if not station.name in self.stations:
             self.stations[station.name] = station
             event.log_event('station_added', self, station)
@@ -68,7 +68,7 @@ class RadioManager(providers.ProviderHandler):
         """
         providers.unregister(self.servicename, station)
 
-    def on_del_station(self, station):
+    def on_provider_removed(self, station):
         if station.name in self.stations:
             del self.stations[station.name]
             event.log_event('station_removed', self, station)
