@@ -36,6 +36,7 @@ from ConfigParser import (
 )
 import logging
 import os
+import sys
 
 import glib
 
@@ -318,6 +319,16 @@ class SettingsManager(RawConfigParser):
         
 location = xdg.get_config_dir()
 
+
+# Provide a mechanism for setting up default settings for different platforms
+if sys.platform == 'win32':
+    __settings_file = 'settings-win32.ini'
+elif sys.platform == 'darwin':
+    __settings_file = 'settings-osx.ini'    
+else:
+    __settings_file = 'settings.ini'
+
+
 MANAGER = SettingsManager(
     os.path.join(location, "settings.ini"),
     xdg.get_config_path("settings.ini")
