@@ -331,8 +331,9 @@ class Track(object):
             # error writing to the file, probably
             logger.warning( "Could not write tags to file: %s" % e )
             return False
-        except Exception:
+        except Exception, e:
             common.log_exception(logger)
+            logger.warning( "Unknown exception: Could not write tags to file: %s" % e )
             return False
 
     def read_tags(self):
@@ -492,15 +493,17 @@ class Track(object):
         """
         if tag == '__basename':
             value = self.get_basename()
+        elif tag == '__startoffset': # necessary?
+            value = self.__tags.get(tag, 0)
         else:
             value = self.__tags.get(tag)
 
-        if join and value:
+        if join and value and not tag.startswith('__'):
             return self.join_values(value)
 
         return value
 
-    def get_tag_sort(self, tag, join=True, artist_compilations=True,
+    def get_tag_sort(self, tag, join=True, artist_compilations=False,
             extend_title=True):
         """
             Get a tag value in a form suitable for sorting.
@@ -510,7 +513,7 @@ class Track(object):
                 single value.
             :param artist_compilations: If True, automatically handle
                 albumartist and other compilations detections when
-                tag=="artist".
+                tag=="albumartist".
             :param extend_title: If the title tag is unknown, try to
                 add some identifying information to it.
         """
@@ -519,9 +522,9 @@ class Track(object):
         # values.
         value = None
         sorttag = self.__tags.get(tag + "sort")
-        if sorttag and tag != "artist":
+        if sorttag and tag != "albumartist":
             value = sorttag
-        elif tag == "artist":
+        elif tag == "albumartist":
             if artist_compilations and self.__tags.get('__compilation'):
                 value = self.__tags.get('albumartist',
                         u"\uffff\uffff\uffff\ufffe")
@@ -565,7 +568,7 @@ class Track(object):
 
         return value
 
-    def get_tag_display(self, tag, join=True, artist_compilations=True,
+    def get_tag_display(self, tag, join=True, artist_compilations=False,
             extend_title=True):
         """
             Get a tag value in a form suitable for display.
@@ -575,7 +578,7 @@ class Track(object):
                 single value.
             :param artist_compilations: If True, automatically handle
                 albumartist and other compilations detections when
-                tag=="artist".
+                tag=="albumartist".
             :param extend_title: If the title tag is unknown, try to
                 add some identifying information to it.
         """
@@ -584,15 +587,15 @@ class Track(object):
             return uri.decode('utf-8')
 
         value = None
-        if tag == "artist":
+        if tag == "albumartist":
             if artist_compilations and self.__tags.get('__compilation'):
                 value = self.__tags.get('albumartist', _VARIOUSARTISTSSTR)
             else:
                 value = self.__tags.get('artist', _UNKNOWNSTR)
         elif tag in ('tracknumber', 'discnumber'):
             value = self.split_numerical(self.__tags.get(tag))[0]
-        elif tag == '__length':
-            value = self.__tags.get('__length', 0)
+        elif tag in ('__length', '__startoffset', '__stopoffset'):
+            value = self.__tags.get(tag, 0)
         elif tag == '__bitrate':
             try:
                 value = int(self.__tags['__bitrate']) // 1000
@@ -632,7 +635,7 @@ class Track(object):
 
         return value
 
-    def get_tag_search(self, tag, format=True, artist_compilations=True,
+    def get_tag_search(self, tag, format=True, artist_compilations=False,
             extend_title=True):
         """
             Get a tag value suitable for passing to the search system.
@@ -641,21 +644,21 @@ class Track(object):
             :param format: pre-format into a search query.
             :param artist_compilations: If True, automatically handle
                 albumartist and other compilations detections when
-                tag=="artist".
+                tag=="albumartist".
             :param extend_title: If the title tag is unknown, try to
                 add some identifying information to it.
         """
         extraformat = ""
-        if tag == "artist":
+        if tag == "albumartist":
             if artist_compilations and self.__tags.get('__compilation'):
-                value = self.__tags.get('albumartist', '__null__')
+                value = self.__tags.get('albumartist', None)
                 tag = 'albumartist'
                 extraformat += " ! __compilation==__null__"
             else:
                 value = self.__tags.get('artist')
         elif tag in ('tracknumber', 'discnumber'):
             value = self.split_numerical(self.__tags.get(tag))[0]
-        elif tag in ('__length', '__playcount', '__rating'):
+        elif tag in ('__length', '__playcount', '__rating', '__startoffset', '__stopoffset'):
             value = self.__tags.get(tag, 0)
         elif tag == '__bitrate':
             try:
@@ -910,7 +913,12 @@ class Track(object):
         if data == "collection/strip_list":
             cls._Track__the_cuts = settings.get_option('collection/strip_list', [])
 
-
+    ### Utility method intended for TrackDB ###
+    
+    @classmethod
+    def _get_track_count(cls):
+        '''Internal API, returns number of track objects we have'''
+        return len(cls._Track__tracksdict)
 
 event.add_callback(Track._the_cuts_cb, 'collection_option_set')
 
