@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -24,7 +24,14 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import os, glib
+import os, sys
+import glib
+
+# We need the local hack for OSX bundled apps, so we depend on the main script
+# to set the environment variable correctly instead of trying to infer an
+# absolute path
+#exaile_dir = os.path.split(os.path.dirname(os.path.realpath(__file__)))[0]
+exaile_dir = os.environ['EXAILE_DIR']
 
 homedir = os.path.expanduser("~")
 lastdir = homedir
@@ -39,19 +46,26 @@ cache_home = glib.get_user_cache_dir()
 cache_home = os.path.join(cache_home, "exaile")
 
 data_dirs = os.getenv("XDG_DATA_DIRS")
-if data_dirs == None:
-    data_dirs = "/usr/local/share/:/usr/share/"
-data_dirs = [os.path.join(d, "exaile") for d in data_dirs.split(":")]
+if data_dirs is None:
+    if sys.platform == 'win32':
+        data_dirs = [exaile_dir]
+    else:
+        data_dirs = ["/usr/local/share/exaile", "/usr/share/exaile"]
+else:
+    data_dirs = [os.path.join(d, "exaile") for d in data_dirs.split(os.pathsep)]
 
 config_dirs = os.getenv("XDG_CONFIG_DIRS")
-if config_dirs == None:
-    config_dirs = "/etc/xdg"
-config_dirs = [os.path.join(d, "exaile") for d in config_dirs.split(":")]
+if config_dirs is None:
+    if sys.platform == 'win32':
+        config_dirs = [exaile_dir]
+    else:
+        config_dirs = ["/etc/xdg/exaile"]
+else:
+    config_dirs = [os.path.join(d, "exaile") for d in config_dirs.split(os.pathsep)]
 
-exaile_dir = os.path.split(os.path.dirname(os.path.realpath(__file__)))[0]
 local_hack = False
 # Detect if Exaile is not installed.
-if os.path.exists(os.path.join(exaile_dir, 'Makefile')):
+if os.path.exists(os.path.join(exaile_dir, 'data')):
     local_hack = True
     # Insert the "data" directory to data_dirs.
     data_dir = os.path.join(exaile_dir, 'data')
