@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -24,6 +24,14 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+"""
+    D-Bus interface for playback control, data query and others
+
+    Access through the ``/org/exaile/Exaile`` object which
+    implements the ``org.exaile.Exaile`` interface
+"""
+
+from collections import namedtuple
 import logging
 from optparse import OptionParser
 import os
@@ -32,8 +40,12 @@ import traceback
 
 import dbus
 import dbus.service
+import gio
 import gobject
 
+# Be VERY careful what you import here! This module gets loaded even if
+# we are just issuing a dbus command to a running instance, so we need
+# to keep imports as light as possible.
 from xl import event
 from xl.nls import gettext as _
 
@@ -70,20 +82,26 @@ def check_exit(options, args):
                 # This enables:    find PATH -name *.mp3 | exaile -
                 if args[0] == '-':
                     args = sys.stdin.read().split('\n')
-                args = [ os.path.abspath(arg) for arg in args ]
+                args = [ gio.File(arg).get_uri() for arg in args ]
                 iface.Enqueue(args)
 
     if not iface:
         for command in ['GetArtist', 'GetTitle', 'GetAlbum', 'GetLength',
                 'GetRating', 'SetRating', 'IncreaseVolume', 'DecreaseVolume',
-                'Play', 'Stop', 'Next', 'Prev', 'PlayPause',
+                'Play', 'Pause', 'Stop', 'Next', 'Prev', 'PlayPause',
                 'StopAfterCurrent', 'GuiToggleVisible', 'CurrentPosition',
-                'CurrentProgress', 'GetVolume', 'Query']:
+                'CurrentProgress', 'GetVolume', 'Query', 'FormatQuery']:
             if getattr(options, command):
-                return True # exit if a control option is passed instead of 
-                            # starting exaile.
-        return False
+                return "command"
+        return "continue"
 
+    run_commands(options, iface)
+    return "exit"
+
+def run_commands(options, iface):
+    """
+        Actually invoke any commands passed in.
+    """
     comm = False
     info_commands = {
         'GetArtist': 'artist',
@@ -101,52 +119,58 @@ def check_exit(options, args):
                 print value
             comm = True
 
-    modify_commands = [
-        'SetRating'
-    ]
+    argument_commands = (
+        'IncreaseVolume',
+        'DecreaseVolume',
+        'SetRating',
+        'Add',
+        'ExportPlaylist'
+    )
 
-    for command in modify_commands:
+    for command in argument_commands:
         argument = getattr(options, command)
         if argument is not None:
-            getattr(iface, command)(argument)
+            if command in ('IncreaseVolume', 'DecreaseVolume'):
+                iface.ChangeVolume(argument if command == 'IncreaseVolume' else -argument)
+            else:
+                print getattr(iface, command)(argument)
+
             comm = True
 
-    volume_commands = (
-            'IncreaseVolume',
-            'DecreaseVolume',
-            )
-
-    for command in volume_commands:
-        value = getattr(options, command)
-        if value:
-            if command == 'DecreaseVolume': value = -value
-            iface.ChangeVolume(value)
+    # Special handling for FormatQuery & FormatQueryTags
+    format = options.FormatQuery
+    if format is not None:
+        print iface.FormatQuery(format, options.FormatQueryTags or 'title,artist,album,__length')
+        comm = True
 
     run_commands = (
-            'Play',
-            'Stop',
-            'Next',
-            'Prev',
-            'PlayPause',
-            'StopAfterCurrent',
-            'GuiToggleVisible'
-            )
+        'Play',
+        'Pause',
+        'Stop',
+        'Next',
+        'Prev',
+        'PlayPause',
+        'StopAfterCurrent',
+        'GuiToggleVisible',
+        'ToggleMute'
+    )
+
     for command in run_commands:
         if getattr(options, command):
             getattr(iface, command)()
             comm = True
 
     query_commands = (
-            'CurrentPosition',
-            'CurrentProgress',
-            'GetRating',
-            'GetVolume',
-            'Query',
-            )
+        'CurrentPosition',
+        'CurrentProgress',
+        'GetRating',
+        'GetVolume',
+        'Query'
+    )
 
     for command in query_commands:
         if getattr(options, command):
-            print getattr(iface, command)()
+            print getattr(iface, command)(argument)
             comm = True
 
     to_implement = (
@@ -157,7 +181,13 @@ def check_exit(options, args):
             logger.warning("FIXME: command not implemented")
             comm = True
 
-    return True
+    if not comm:
+        iface.GuiToggleVisible()
+
+PlaybackStatus = namedtuple(
+    'PlaybackStatus',
+    'state progress position current'
+)
 
 class DbusManager(dbus.service.Object):
     """
@@ -174,44 +204,58 @@ class DbusManager(dbus.service.Object):
         dbus.service.Object.__init__(self, self.bus_name, '/org/exaile/Exaile')
         self.cached_track = ""
         self.cached_state = ""
+        self.cached_volume = -1
 
     def _connect_signals(self):
         # connect events
-        event.add_callback(self.emit_state_changed, 'playback_player_end',
-            self.exaile.player)
-        event.add_callback(self.emit_state_changed, 'playback_track_start',
-            self.exaile.player)
-        event.add_callback(self.emit_state_changed, 'playback_toggle_pause',
-            self.exaile.player)
-        event.add_callback(self.emit_track_changed, 'tags_parsed',
-            self.exaile.player)
-        event.add_callback(self.emit_state_changed, 'playback_buffering',
-            self.exaile.player)
-        event.add_callback(self.emit_state_changed, 'playback_error',
-            self.exaile.player)
+        from xl import player
+        event.add_callback(self.emit_state_changed,
+            'playback_player_end', player.PLAYER)
+        event.add_callback(self.emit_state_changed,
+            'playback_track_start', player.PLAYER)
+        event.add_callback(self.emit_state_changed,
+            'playback_toggle_pause', player.PLAYER)
+        event.add_callback(self.emit_track_changed,
+            'tags_parsed', player.PLAYER)
+        event.add_callback(self.emit_state_changed,
+            'playback_buffering', player.PLAYER)
+        event.add_callback(self.emit_state_changed,
+            'playback_error', player.PLAYER)
 
 
     @dbus.service.method('org.exaile.Exaile', 's')
     def TestService(self, arg):
         """
             Just test the dbus object
+
+            :param arg: anything
         """
         logger.debug(arg)
 
     @dbus.service.method('org.exaile.Exaile', None, 'b')
     def IsPlaying(self):
         """
-            Returns True if Exaile is playing (or paused), False if it's not
+            Determines if Exaile is playing / paused or not
+
+            :returns: whether Exaile is playing / paused
+            :rtype: boolean
         """
-        return bool(self.exaile.player.current)
+        from xl import player
+        return bool(not player.PLAYER.is_stopped())
 
     @dbus.service.method('org.exaile.Exaile', 's', 's')
     def GetTrackAttr(self, attr):
         """
-            Returns an attribute of a track
+            Returns the value of a track tag
+
+            :param attr: a track tag
+            :type attr: string
+            :returns: the tag value
+            :rtype: string
         """
+        from xl import player
         try:
-            value = self.exaile.player.current.get_tag_raw(attr)
+            value = player.PLAYER.current.get_tag_raw(attr) or ''
         except (ValueError, TypeError, AttributeError):
             value = ''
 
@@ -222,10 +266,16 @@ class DbusManager(dbus.service.Object):
     @dbus.service.method('org.exaile.Exaile', 'sv')
     def SetTrackAttr(self, attr, value):
         """
-            Sets rating of a track
+            Sets the value of a track tag
+
+            :param attr: a track tag
+            :type attr: string
+            :param value: the tag value
+            :type value: any
         """
+        from xl import player
         try:
-            self.exaile.player.current.set_tag_raw(attr, value)
+            player.PLAYER.current.set_tag_raw(attr, value)
         except (AttributeError, TypeError):
             pass
 
@@ -233,9 +283,12 @@ class DbusManager(dbus.service.Object):
     def GetRating(self):
         """
             Returns the current track's rating
+
+            :returns: the rating
+            :rtype: int
         """
         try:
-            rating = int(self.GetTrackAttr('__rating'))
+            rating = int(float(self.GetTrackAttr('__rating')))
         except ValueError:
             rating = 0
 
@@ -245,77 +298,118 @@ class DbusManager(dbus.service.Object):
     def SetRating(self, value):
         """
             Sets the current track's rating
+
+            :param value: the new rating
+            :type value: int
         """
         self.SetTrackAttr('__rating', value)
+        event.log_event('rating_changed', self, value)
 
     @dbus.service.method('org.exaile.Exaile', 'i')
     def ChangeVolume(self, value):
         """
             Changes volume by the specified amount (in percent, can be negative)
+
+            :param value: the new volume
+            :type value: int
         """
-        player = self.exaile.player
-        player.set_volume(player.get_volume() + value)
+        from xl import player
+        player.PLAYER.set_volume(player.PLAYER.get_volume() + value)
+        self.cached_volume = -1
+
+    @dbus.service.method('org.exaile.Exaile')
+    def ToggleMute(self):
+        """
+            Mutes or unmutes the volume
+        """
+        from xl import player
+        if self.cached_volume == -1:
+            self.cached_volume = player.PLAYER.get_volume()
+            player.PLAYER.set_volume(0)
+        else:
+            player.PLAYER.set_volume(self.cached_volume)
+            self.cached_volume = -1
 
     @dbus.service.method('org.exaile.Exaile', 'd')
     def Seek(self, value):
         """
             Seeks to the given position in seconds
+
+            :param value: the position in seconds
+            :type value: int
         """
-        player = self.exaile.player
-        player.seek(value)
+        from xl import player
+        player.PLAYER.seek(value)
 
     @dbus.service.method('org.exaile.Exaile')
     def Prev(self):
         """
             Jumps to the previous track
         """
-        self.exaile.queue.prev()
+        from xl import player
+        player.QUEUE.prev()
 
     @dbus.service.method('org.exaile.Exaile')
     def Stop(self):
         """
             Stops playback
         """
-        self.exaile.player.stop()
+        from xl import player
+        player.PLAYER.stop()
 
     @dbus.service.method('org.exaile.Exaile')
     def Next(self):
         """
             Jumps to the next track
         """
-        self.exaile.queue.next()
+        from xl import player
+        player.QUEUE.next()
 
     @dbus.service.method('org.exaile.Exaile')
     def Play(self):
         """
             Starts playback
         """
-        self.exaile.queue.play()
+        from xl import player
+        player.QUEUE.play()
+
+    @dbus.service.method('org.exaile.Exaile')
+    def Pause(self):
+        """
+            Starts playback
+        """
+        from xl import player
+        player.PLAYER.pause()
 
     @dbus.service.method('org.exaile.Exaile')
     def PlayPause(self):
         """
             Toggle Play or Pause
         """
-        if self.exaile.player.get_state() == 'stopped':
-            self.exaile.player.play(self.exaile.queue.get_current())
+        from xl import player
+        if player.PLAYER.is_stopped():
+            player.PLAYER.play(player.QUEUE.get_current())
         else:
-            self.exaile.player.toggle_pause()
+            player.PLAYER.toggle_pause()
 
     @dbus.service.method('org.exaile.Exaile')
     def StopAfterCurrent(self):
         """
             Toggle stopping after current track
         """
-        current_track = self.exaile.queue.get_current()
-        self.exaile.queue.stop_track = current_track
+        from xl import player
+        player.QUEUE.stop_track = player.QUEUE.get_current()
 
     @dbus.service.method('org.exaile.Exaile', None, 's')
     def CurrentProgress(self):
         """
             Returns the progress into the current track (in percent)
+
+            :returns: the current progress
+            :rtype: string
         """
-        progress = self.exaile.player.get_progress()
+        from xl import player
+        progress = player.PLAYER.get_progress()
         if progress == -1:
             return ""
         return str(int(progress * 100))
@@ -324,48 +418,128 @@ class DbusManager(dbus.service.Object):
     def CurrentPosition(self):
         """
             Returns the position inside the current track (as time)
+
+            :returns: the current position
+            :rtype: string
         """
-        progress = self.exaile.player.get_time()
+        from xl import player
+        progress = player.PLAYER.get_time()
         return '%d:%02d' % (progress // 60, progress % 60)
 
     @dbus.service.method('org.exaile.Exaile', None, 's')
     def GetVolume(self):
         """
             Returns the current volume level (in percent)
+
+            :returns: the current volume
+            :rtype: string
         """
-        return str(self.exaile.player.get_volume())
+        from xl import player
+        return str(player.PLAYER.get_volume())
+
+    def __get_playback_status(self, tags=["title", "artist", "album", "__length"]):
+        """
+            Retrieves the playback status
+
+            :param tags: tags to retrieve from the current track
+            :type tags: list
+            :returns: the playback status
+            :rtype: :class:`xl.xldbus.PlaybackStatus`
+        """
+        status = PlaybackStatus(
+            state='stopped',
+            current=None,
+            progress=0,
+            position=0
+        )
+
+        from xl import player
+
+        current_track = player.QUEUE.get_current()
+        progress = player.PLAYER.get_progress()
+
+        if current_track is not None and \
+           not player.PLAYER.is_stopped():
+            current = {}
+            # Make tags unique
+            tags = list(set(tags))
+
+            for tag in tags:
+                current[tag] = self.GetTrackAttr(tag)
+
+            # Special handling for internal tags
+            if "__length" in tags:
+                from xl.formatter import LengthTagFormatter
+                current["__length"] = LengthTagFormatter.format_value(self.GetTrackAttr('__length'))
+            
+            status = PlaybackStatus(
+                state=player.PLAYER.get_state(),
+                progress=self.CurrentProgress(),
+                position=self.CurrentPosition(),
+                current=current
+            )
+
+        return status
+
 
     @dbus.service.method('org.exaile.Exaile', None, 's')
     def Query(self):
         """
             Returns information about the currently playing track
-        """
-        current_track = self.exaile.queue.get_current()
-        if current_track is None or not \
-           (self.exaile.player.is_playing() or self.exaile.player.is_paused()):
-            return _('Not playing.')
 
-        length = float(self.GetTrackAttr('__length'))
-        length = '%d:%02d' % (length // 60, length % 60)
+            :returns: information about the current track
+            :rtype: string
+        """
+        status = self.__get_playback_status()
+
+        if status.current is None or status.state == "stopped":
+            return _('Not playing.')
 
         result = _('status: %(status)s, title: %(title)s, artist: %(artist)s,'
                    ' album: %(album)s, length: %(length)s,'
                    ' position: %(progress)s%% [%(position)s]') % {
-                         'status': self.exaile.player.get_state(),
-                         'title': self.GetTrackAttr('title'),
-                         'artist': self.GetTrackAttr('artist'),
-                         'album': self.GetTrackAttr('album'),
-                         'length': length,
-                         'progress': self.CurrentProgress(),
-                         'position': self.CurrentPosition(),
+                         'status': status.state,
+                         'title': status.current["title"],
+                         'artist': status.current["artist"],
+                         'album': status.current["album"],
+                         'length': status.current["__length"],
+                         'progress': status.progress,
+                         'position': status.position,
                      }
 
         return result
 
+    @dbus.service.method('org.exaile.Exaile', 'ss', 's')
+    def FormatQuery(self, format, tags):
+        """
+            Returns the current playback state including
+            information about the currently playing track
+
+            :param format: the desired output format (currently only "json")
+            :type format: string
+            :param tags: tags to retrieve from the current track
+            :type tags: string
+            :returns: the formatted information or an empty
+                      string if requesting an unknown format
+            :rtype: string
+        """
+        # TODO: Elaborate the use of providers here
+        if format == 'json':
+            import json
+
+            status = self.__get_playback_status(tags.split(","))
+
+            return json.dumps(status._asdict())
+
+        return ''
+
     @dbus.service.method('org.exaile.Exaile', None, 's')
     def GetVersion(self):
         """
             Returns the version of Exaile
+
+            :returns: the application version
+            :rtype: string
         """
         return self.exaile.get_version()
 
@@ -373,66 +547,85 @@ class DbusManager(dbus.service.Object):
     def PlayFile(self, filename):
         """
             Plays the specified file
+
+            :param filename: the path to start playing
+            :type filename: string
         """
         self.exaile.gui.open_uri(filename)
 
     @dbus.service.method('org.exaile.Exaile', 'as')
-    def Enqueue(self, filenames):
+    def Enqueue(self, locations):
         """
-            Adds the specified files to the current playlist
+            Adds the tracks at the specified locations
+            to the current playlist
+
+            :param locations: locations to enqueue
+            :type locations: iterable
         """
-        print filenames
-        import xl.playlist
-        from xl import trax   # do this here to avoid loading
-                              # settings when issuing dbus commands
-        # FIXME: Get rid of dependency on xlgui
-        #        by moving sorting column somewhere else
-        pl = self.exaile.gui.main.get_selected_playlist()
-        column, descending = pl.get_sort_by()
-        tracks = []
-        playlists = []
+        from xl import player
+        from xlgui import get_controller
 
-        for file in filenames:
-            if xl.playlist.is_valid_playlist(file):
-                try:
-                    pl = xl.playlist.import_playlist(file)
-                    tracks += pl.get_tracks()
-                    continue
-                except xl.playlist.InvalidPlaylistTypeException:
-                    pass
-                except:
-                    traceback.print_exc()
-            else:
-                tracks += trax.get_tracks_from_uri(file)
+        controller = get_controller()
 
-        print tracks
+        play = False
 
-        if column:
-            tracks = trax.sort_tracks([column], tracks)
-        self.exaile.queue.current_playlist.add_tracks(tracks)
+        # Allows for playing first item
+        if player.PLAYER.is_stopped():
+            play = True
 
-        if not self.exaile.player.is_playing():
-            try:
-                pos = self.exaile.queue.current_playlist.index(tracks[0])
-                self.exaile.queue.current_playlist.set_current_pos(pos)
-                self.exaile.queue.play()
-            except IndexError:
-                pass
+        for location in locations:
+            controller.open_uri(location, play=play)
+            play = False
+
+    @dbus.service.method('org.exaile.Exaile', 's')
+    def Add(self, location):
+        """
+            Adds the tracks at the specified
+            location to the collection
+
+            :param location: where to add tracks from
+            :type location: string
+        """
+        from xl import trax
+
+        tracks = trax.get_tracks_from_uri(location)
+        self.exaile.collection.add_tracks(tracks)
+
+    @dbus.service.method('org.exaile.Exaile', 's')
+    def ExportPlaylist(self, location):
+        """
+            Exports the current playlist
+            to the specified location
+
+            :param location: where to save the playlist at
+            :type location: string
+        """
+        from xl import player, playlist
+
+        if player.QUEUE.current_playlist is not None:
+            playlist.export_playlist(
+                player.QUEUE.current_playlist,
+                location
+            )
 
     @dbus.service.method('org.exaile.Exaile')
     def GuiToggleVisible(self):
         """
             Toggles visibility of the GUI, if possible
         """
-        self.exaile.gui.main.toggle_visible()
+        self.exaile.gui.main.toggle_visible(bringtofront=True)
 
     @dbus.service.method('org.exaile.Exaile', None, 'ay')
     def GetCoverData(self):
         """
             Returns the data of the cover image of the playing track, or
             an empty string if there is no cover available.
+
+            :returns: the cover data
+            :rtype: binary data
         """
-        cover = self.exaile.covers.get_cover(self.exaile.player.current)
+        from xl import covers, player
+        cover = covers.MANAGER.get_cover(player.PLAYER.current, set_only=True)
         if not cover:
             cover = ''
         return cover
@@ -441,8 +634,12 @@ class DbusManager(dbus.service.Object):
     def GetState(self):
         """
             Returns the surrent verbatim state (unlocalized)
+
+            :returns: the player state
+            :rtype: string
         """
-        return self.exaile.player.get_state()
+        from xl import player
+        return player.PLAYER.get_state()
 
     @dbus.service.signal('org.exaile.Exaile')
     def StateChanged(self):
@@ -462,7 +659,7 @@ class DbusManager(dbus.service.Object):
         """
             Called from main to emit signal
         """
-        new_state = self.exaile.player.get_state()
+        new_state = player.get_state()
         if self.cached_state != new_state:
             self.cached_state = new_state
             self.StateChanged()
@@ -476,3 +673,4 @@ class DbusManager(dbus.service.Object):
             self.cached_track = new_track
             self.TrackChanged()
 
+# vim: et sts=4 sw=4
