@@ -55,7 +55,7 @@ from xl import (
     xdg
 )
 from xlgui.accelerators import AcceleratorManager
-from xlgui.playlist import PlaylistNotebook
+from xlgui.playlist_container import PlaylistContainer
 from xlgui.widgets import (
     dialogs,
     info,
@@ -67,7 +67,6 @@ from xlgui.widgets.playlist import (
     PlaylistView
 )
 from xlgui import (
-    cover,
     guiutil,
     tray,
     menu as mainmenu
@@ -102,6 +101,9 @@ class MainWindow(gobject.GObject):
         self.current_page = -1
         self._fullscreen = False
         self.resuming = False
+        
+        self.window_state = 0
+        self.minimized = False
 
         self.builder = builder
 
@@ -166,7 +168,7 @@ class MainWindow(gobject.GObject):
             ('<Control>Page_Up', self._on_prev_tab_key),
             ('<Control>Page_Down', self._on_next_tab_key),
             
-            ('<Alt>N', self._on_focus_playlist_notebook),
+            ('<Alt>N', self._on_focus_playlist_container),
 
             # These 4 are subject to change.. probably should do this 
             # via a different mechanism too... 
@@ -205,17 +207,13 @@ class MainWindow(gobject.GObject):
         )
         self.message.connect('response', self.on_messagebar_response)
 
-        self.info_area = info.TrackInfoPane(player.PLAYER)
+        self.info_area = MainWindowTrackInfoPane(player.PLAYER)
         self.info_area.set_auto_update(True)
         self.info_area.set_padding(3, 3, 3, 3)
         self.info_area.hide_all()
         self.info_area.set_no_show_all(True)
         guiutil.gtk_widget_replace(self.builder.get_object('info_area'), self.info_area)
 
-        self.cover = cover.CoverWidget(self.info_area.cover_image, player.PLAYER)
-        self.cover.hide_all()
-        self.cover.set_no_show_all(True)
-
         self.volume_control = playback.VolumeControl(player.PLAYER)
         self.info_area.get_action_area().pack_start(self.volume_control)
 
@@ -231,14 +229,15 @@ class MainWindow(gobject.GObject):
                 self.window.connect('screen-changed', self.on_screen_changed)
 
         playlist_area = self.builder.get_object('playlist_area')
-        self.playlist_notebook = PlaylistNotebook('saved_tabs', player.PLAYER)
-        self.playlist_notebook.connect_after('switch-page',
-            self.on_playlist_notebook_switch_page)
-        playlist_area.pack_start(self.playlist_notebook, padding=3)
-        page_num = self.playlist_notebook.get_current_page()
-        page = self.playlist_notebook.get_nth_page(page_num)
-        selection = page.view.get_selection()
-        selection.connect('changed', self.on_playlist_view_selection_changed)
+        self.playlist_container = PlaylistContainer('saved_tabs', player.PLAYER)
+        for notebook in self.playlist_container.notebooks:
+            notebook.connect_after('switch-page', self.on_playlist_container_switch_page)
+            page = notebook.get_current_tab()
+            if page is not None:
+                selection = page.view.get_selection()
+                selection.connect('changed', self.on_playlist_view_selection_changed)
+            
+        playlist_area.pack_start(self.playlist_container, padding=3)
 
         self.splitter = self.builder.get_object('splitter')
 
@@ -296,7 +295,6 @@ class MainWindow(gobject.GObject):
             # Controller
 #            'on_scan_collection_item_activate': self.controller.on_rescan_collection,
 #            'on_device_manager_item_activate': lambda *e: self.controller.show_devices(),
-            'on_panel_notebook_switch_page': self.controller.on_panel_switch,
 #            'on_track_properties_activate':self.controller.on_track_properties,
         })
 
@@ -332,11 +330,18 @@ class MainWindow(gobject.GObject):
         """
             Sets up panel events
         """
+        
+        self.panel_box = self.builder.get_object('panel_box')
+        
+        # When there's nothing in the notebook, hide it
+        self.controller.panel_notebook.connect('page-added', self.on_panel_notebook_add_page)
+        self.controller.panel_notebook.connect('page-removed', self.on_panel_notebook_remove_page)
+        
         # panels
-        panels = self.controller.panels
+        panels = self.controller.panel_notebook.panels
 
         for panel_name in ('playlists', 'radio', 'files', 'collection'):
-            panel = panels[panel_name]
+            panel = panels[panel_name].panel
             sort = False
 
             if panel_name in ('files', 'collection'):
@@ -350,23 +355,23 @@ class MainWindow(gobject.GObject):
                 self.on_append_items(items, replace=True, sort=sort))
 
         ## Collection Panel
-        panel = panels['collection']
+        panel = panels['collection'].panel
         panel.connect('collection-tree-loaded', self.on_collection_tree_loaded)
 
         ## Playlist Panel
-        panel = panels['playlists']
+        panel = panels['playlists'].panel
         panel.connect('playlist-selected',
             lambda panel, playlist:
-                self.playlist_notebook.create_tab_from_playlist(playlist))
+                self.playlist_container.create_tab_from_playlist(playlist))
 
         ## Radio Panel
-        panel = panels['radio']
+        panel = panels['radio'].panel
         panel.connect('playlist-selected',
             lambda panel, playlist:
-                self.playlist_notebook.create_tab_from_playlist(playlist))
+                self.playlist_container.create_tab_from_playlist(playlist))
 
         ## Files Panel
-        panel = panels['files']
+        #panel = panels['files']
 
     def on_expose_event(self, widget, event):
         """
@@ -390,7 +395,7 @@ class MainWindow(gobject.GObject):
         """
         screen = widget.get_screen()
         colormap = screen.get_rgba_colormap() or screen.get_rgb_colormap()
-        self.window.set_colormap(rgbamap)
+        self.window.set_colormap(colormap)
 
     def on_messagebar_response(self, widget, response):
         """
@@ -398,6 +403,14 @@ class MainWindow(gobject.GObject):
         """
         if response == gtk.RESPONSE_CLOSE:
             widget.hide()
+            
+    def on_panel_notebook_add_page(self, notebook, page, page_num):
+        if self.splitter.get_child1() is None:
+            self.splitter.pack1(self.panel_box)
+        
+    def on_panel_notebook_remove_page(self, notebook, page, page_num):
+        if notebook.get_n_pages() == 0:
+            self.splitter.remove(self.panel_box)
 
     def on_stop_button_motion_notify_event(self, widget, event):
         """
@@ -522,7 +535,7 @@ class MainWindow(gobject.GObject):
             player.QUEUE.current_playlist.spat_position = -1
 
         self.get_selected_page().view.queue_draw()
-
+    
     def on_append_items(self, tracks, force_play=False, queue=False, sort=False, replace=False):
         """
             Called when a panel (or other component)
@@ -537,7 +550,7 @@ class MainWindow(gobject.GObject):
             :param sort: Sort before adding
             :param replace: Clear playlist before adding
         """
-        if not tracks:
+        if len(tracks) == 0:
             return
 
         page = self.get_selected_page()
@@ -638,18 +651,11 @@ class MainWindow(gobject.GObject):
         # refresh the current playlist
         pl = self.get_selected_page()
 
-
-    def on_collection_tree_loaded(self, tree):
-        """
-            Updates info after collection tree load
-        """
-        self.statusbar.update_info()
-
-    def on_playlist_notebook_switch_page(self, notebook, page, page_num):
+    def on_playlist_container_switch_page(self, notebook, page, page_num):
         """
             Updates info after notebook page switch
         """
-        page = self.playlist_notebook.get_nth_page(page_num)
+        page = notebook.get_nth_page(page_num)
         selection = page.view.get_selection()
         selection.connect('changed', self.on_playlist_view_selection_changed)
         self.statusbar.update_info()
@@ -664,9 +670,8 @@ class MainWindow(gobject.GObject):
         """
             Gives focus to the filter field of the current panel
         """
-        panel_name = settings.get_option('gui/last_selected_panel', 'collection')
         try:
-            self.controller.panels[panel_name].filter.grab_focus()
+            self.controller.get_active_panel().filter.grab_focus()
         except (AttributeError, KeyError):
             pass
 
@@ -790,7 +795,7 @@ class MainWindow(gobject.GObject):
         """
             Tries to show the currently playing track
         """
-        self.playlist_notebook.show_current_track()
+        self.playlist_container.show_current_track()
 
     def on_about_item_activate(self, menuitem):
         """
@@ -857,12 +862,13 @@ class MainWindow(gobject.GObject):
             
         if option == 'gui/show_info_area_covers':
             def _setup_info_covers():
-                self.cover.set_no_show_all(False)
+                cover = self.info_area.cover
+                cover.set_no_show_all(False)
                 if settings.get_option(option, True):
-                    self.cover.show_all()
+                    cover.show_all()
                 else:
-                    self.cover.hide_all()
-                self.cover.set_no_show_all(True)
+                    cover.hide_all()
+                cover.set_no_show_all(True)
                 
             glib.idle_add(_setup_info_covers)
 
@@ -884,11 +890,11 @@ class MainWindow(gobject.GObject):
         return True
 
     def _on_prev_tab_key(self, *e):
-        self.playlist_notebook.select_prev_tab()
+        self.playlist_container.get_current_notebook().select_prev_tab()
         return True
     
     def _on_next_tab_key(self, *e):
-        self.playlist_notebook.select_next_tab()
+        self.playlist_container.get_current_notebook().select_next_tab()
         return True
     
     def _on_playpause_button(self, *e):
@@ -896,12 +902,11 @@ class MainWindow(gobject.GObject):
         return True
 
     def _on_focus_playlist_tab(self, tab_nr):
-        if tab_nr < self.playlist_notebook.get_n_pages():
-            self.playlist_notebook.focus_tab(tab_nr)
+        self.playlist_container.get_current_notebook().focus_tab(tab_nr)
         return True
     
-    def _on_focus_playlist_notebook(self, *_e):
-        self.playlist_notebook.focus()
+    def _on_focus_playlist_container(self, *_e):
+        self.playlist_container.focus()
         return True
 
     def _update_track_information(self):
@@ -994,8 +999,10 @@ class MainWindow(gobject.GObject):
                not bringtofront and self.window.get_property('visible'):
                 self.window.hide()
             else:
-                self.window.show()
+                # the ordering for deiconify/show matters -- if this gets 
+                # switched, then the minimization detection breaks
                 self.window.deiconify()
+                self.window.show()
 
     def configure_event(self, *e):
         """
@@ -1030,20 +1037,51 @@ class MainWindow(gobject.GObject):
         if event.changed_mask & gtk.gdk.WINDOW_STATE_FULLSCREEN:
             self._fullscreen = bool(event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN)
 
-        if settings.get_option('gui/minimize_to_tray', False):
-            wm_state = window.window.property_get('_NET_WM_STATE')
+        # detect minimization state changes
+        prev_minimized = self.minimized
+        
+        if not self.minimized:
+            
+            if event.changed_mask & gtk.gdk.WINDOW_STATE_ICONIFIED and \
+               not event.changed_mask & gtk.gdk.WINDOW_STATE_WITHDRAWN and \
+               event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED and \
+               not event.new_window_state & gtk.gdk.WINDOW_STATE_WITHDRAWN and \
+               not self.window_state & gtk.gdk.WINDOW_STATE_ICONIFIED:
+                
+                self.minimized = True
+        else:
+            if event.changed_mask & gtk.gdk.WINDOW_STATE_WITHDRAWN and \
+               not event.new_window_state & (gtk.gdk.WINDOW_STATE_WITHDRAWN): #and \
+                
+                self.minimized = False
 
-            if wm_state is not None:
-                if '_NET_WM_STATE_HIDDEN' in wm_state[2]:
-                    if not settings.get_option('gui/use_tray', False) and \
-                        self.controller.tray_icon is None:
-                        self.controller.tray_icon = tray.TrayIcon(self)
-                    window.hide()
-                else:
-                    if not settings.get_option('gui/use_tray', False) and \
-                        self.controller.tray_icon is not None:
-                        self.controller.tray_icon.destroy()
-                        self.controller.tray_icon = None
+        # track this
+        self.window_state = event.new_window_state
+        
+        if settings.get_option('gui/minimize_to_tray', False):
+            
+            # old code to detect minimization
+            # -> it must have worked at some point, perhaps this is a GTK version
+            # specific set of behaviors? Current code works now on 2.24.17
+             
+            #if wm_state is not None:
+            #    if '_NET_WM_STATE_HIDDEN' in wm_state[2]:
+            #        show tray
+            #        window.hide
+            #else
+            #    destroy tray
+            
+            if self.minimized != prev_minimized and self.minimized == True:
+                if not settings.get_option('gui/use_tray', False) and \
+                    self.controller.tray_icon is None:
+                    self.controller.tray_icon = tray.TrayIcon(self)
+                
+                window.hide()
+            elif window.window.property_get('_NET_WM_STATE') is None:
+                if not settings.get_option('gui/use_tray', False) and \
+                    self.controller.tray_icon is not None:
+                    self.controller.tray_icon.destroy()
+                    self.controller.tray_icon = None
 
         return False
 
@@ -1051,7 +1089,7 @@ class MainWindow(gobject.GObject):
         """
             Returns the currentry displayed playlist notebook page
         """
-        return self.playlist_notebook.get_current_tab()
+        return self.playlist_container.get_current_tab()
     
     def get_selected_playlist(self):
         try:
@@ -1060,10 +1098,65 @@ class MainWindow(gobject.GObject):
             return None
         if not isinstance(page, PlaylistPage):
             return None
-        return page  
+        return page
 
+class MainWindowTrackInfoPane(info.TrackInfoPane, providers.ProviderHandler):
+    """
+        Extends the regular track info pane by an area for custom widgets
+
+        The mainwindow-info-area-widget provider is used to show widgets
+        on the right of the info area. They should be small. The registered
+        provider should provide a method 'create_widget' that takes the info
+        area instance as a parameter, and that returns a gtk.Widget to be
+        inserted into the widget_area of the info area, and an attribute
+        'name' that will be used when removing the provider.
+    """
+    def __init__(self, player):
+        info.TrackInfoPane.__init__(self, player)
+
+        self.__player = player
+        self.widget_area = gtk.HBox()
+
+        self.get_child().pack_start(self.widget_area, expand=False, fill=False)
+
+        self.__widget_area_widgets = {}
+        
+        # call this last if we're using simple_init=True
+        providers.ProviderHandler.__init__(self, 'mainwindow-info-area-widget',
+            target=player, simple_init=True)
+        
+    def get_player(self):
+        '''
+            Retrieves the player object that this info area
+            is associated with
+        '''
+        return self._TrackInfoPane__player
+            
+    def on_provider_added(self, provider):
+        name = provider.name
+        widget = provider.create_widget(self)
+        
+        old_widget = self.__widget_area_widgets.get(name)
+        if old_widget is not None:
+            self.widget_area.remove(old_widget)
+            old_widget.destroy()
+        
+        self.__widget_area_widgets[name] = widget
+        self.widget_area.pack_start(widget, False, False)
+        widget.show_all()
+    
+    def on_provider_removed(self, provider):
+        widget = self.__widget_area_widgets.pop(provider.name, None)
+        if widget is not None:
+            self.widget_area.remove(widget)
+            widget.destroy()
+
+def get_playlist_container():
+    return MainWindow._mainwindow.playlist_container
+        
 def get_playlist_notebook():
-    return MainWindow._mainwindow.playlist_notebook
+    '''Retrieves the primary playlist notebook'''
+    return MainWindow._mainwindow.playlist_container.notebooks[0]
 
 def get_selected_page():
     return MainWindow._mainwindow.get_selected_page()
