@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -26,48 +26,52 @@
 
 import threading
 
+import glib
 import gobject
 import gtk
 
+from xl import common, event
 from xl.nls import gettext as _
-from xl import event
 from xlgui import panel
 from xlgui.panel.collection import CollectionPanel
 from xlgui.panel.flatplaylist import FlatPlaylistPanel
 
-
-class DeviceTransferThread(threading.Thread):
-    def __init__(self, device, main, panel):
-        threading.Thread.__init__(self)
-        self.setDaemon(True)
+class DeviceTransferThread(common.ProgressThread):
+    """
+        Transfers tracks from devices
+    """
+    def __init__(self, device):
+        common.ProgressThread.__init__(self)
 
         self.device = device
-        self.main = main
-        self.panel = panel
 
-    def stop_thread(self):
+    def stop(self):
+        """
+            Stops the thread
+        """
         self.device.transfer.cancel()
+        common.ProgressThread.stop(self)
 
-    def thread_complete(self):
+    def on_track_transfer_progress(self, type, transfer, progress):
         """
-            Called when the thread has finished normally
+            Notifies about progress changes
         """
-        gobject.idle_add(self.panel.load_tree)
-
-    def progress_update(self, type, transfer, progress):
-        event.log_event('progress_update', self, progress)
+        if progress < 100:
+            self.emit('progress-update', progress)
+        else:
+            self.emit('done')
 
     def run(self):
         """
             Runs the thread
         """
-        event.add_callback(self.progress_update, 'track_transfer_progress',
-            self.device.transfer)
+        event.add_callback(self.on_track_transfer_progress,
+            'track_transfer_progress', self.device.transfer)
         try:
             self.device.start_transfer()
         finally:
-            event.remove_callback(self.progress_update, 'track_transfer_progress',
-                self.device.transfer)
+            event.remove_callback(self.on_track_transfer_progress,
+                'track_transfer_progress', self.device.transfer)
 
 class ReceptiveCollectionPanel(CollectionPanel):
     def drag_data_received(self, widget, context, x, y, data, info, stamp):
@@ -95,28 +99,31 @@ class DevicePanel(panel.Panel):
         generic panel for devices
     """
     __gsignals__ = {
-        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object, bool)),
+        'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'collection-tree-loaded': (gobject.SIGNAL_RUN_LAST, None, ()),
     }
 
-    ui_info = ('device_panel.ui', 'DevicePanelWindow')
+    ui_info = ('device.ui', 'DevicePanelWindow')
 
-    def __init__(self, parent, main,
-        device, name=None):
+    def __init__(self, parent, main, device, name):
 
-        panel.Panel.__init__(self, name)
+        label = device.get_name()
+        panel.Panel.__init__(self, parent, name, label)
         self.device = device
         self.main = main
 
         self.notebook = self.builder.get_object("device_notebook")
 
         self.collectionpanel = ReceptiveCollectionPanel(parent,
-            collection=device.collection, name=name)
+            collection=device.collection, name=name, label=label)
         self.collectionpanel.add_tracks_func = self.add_tracks_func
 
         self.collectionpanel.connect('append-items',
             lambda *e: self.emit('append-items', *e[1:]))
+        self.collectionpanel.connect('replace-items',
+            lambda *e: self.emit('replace-items', *e[1:]))
         self.collectionpanel.connect('queue-items',
             lambda *e: self.emit('queue-items', *e[1:]))
         self.collectionpanel.connect('collection-tree-loaded',
@@ -124,9 +131,10 @@ class DevicePanel(panel.Panel):
 
     def add_tracks_func(self, tracks):
         self.device.add_tracks(tracks)
-        thread = DeviceTransferThread(self.device, self.main, self)
+        thread = DeviceTransferThread(self.device)
+        thread.connect('done', lambda *e: self.load_tree())
         self.main.controller.progress_manager.add_monitor(thread,
-                _("Transferring to %s...")%self.name, 'gtk-go-up')
+                _("Transferring to %s...") % self.name, gtk.STOCK_GO_UP)
 
     def get_panel(self):
         return self.collectionpanel.get_panel()
@@ -140,25 +148,28 @@ class DevicePanel(panel.Panel):
 
 class FlatPlaylistDevicePanel(panel.Panel):
     __gsignals__ = {
-        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,bool)),
+        'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
 
-    ui_info = ('device_panel.ui', 'DevicePanelWindow')
+    ui_info = ('device.ui', 'DevicePanelWindow')
 
-    def __init__(self, parent, main,
-        device, name=None):
+    def __init__(self, parent, main, device, name):
 
-        panel.Panel.__init__(self, name)
+        label = device.get_name()
+        panel.Panel.__init__(self, parent, name, label)
         self.device = device
         self.main = main
 
         self.notebook = self.builder.get_object("device_notebook")
 
-        self.fppanel = FlatPlaylistPanel(self, name)
+        self.fppanel = FlatPlaylistPanel(self, name, label)
 
         self.fppanel.connect('append-items',
             lambda *e: self.emit('append-items', *e[1:]))
+        self.fppanel.connect('replace-items',
+            lambda *e: self.emit('replace-items', *e[1:]))
         self.fppanel.connect('queue-items',
             lambda *e: self.emit('queue-items', *e[1:]))
 
