@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -24,40 +24,57 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import os
-import locale
-import re
-import urllib
-
 import gio
 import glib
 import gobject
 import gtk
+import locale
+import logging
+import os
+import pango
+import re
+import urllib
 
-from xl import common, trax, metadata
-from xl import settings
-from xl import event
-from xlgui import guiutil, panel, playlist, menu, xdg
+from xl import (
+    common,
+    event,
+    metadata,
+    providers,
+    settings,
+    trax
+)
 from xl.nls import gettext as _
-import xlgui
+from xlgui import (
+    guiutil,
+    icons,
+    panel,
+    xdg
+)
+
+from xlgui.panel import menus
+from xlgui.widgets.common import DragTreeView
+
+
+logger = logging.getLogger(__name__)
+
 
 class FilesPanel(panel.Panel):
     """
         The Files panel
     """
     __gsignals__ = {
-        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'append-items': (gobject.SIGNAL_RUN_LAST, None, (object, bool)),
         'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
 
-    ui_info = ('files_panel.ui', 'FilesPanelWindow')
+    ui_info = ('files.ui', 'FilesPanelWindow')
 
-    def __init__(self, parent, collection):
+    def __init__(self, parent, collection, name):
         """
             Initializes the files panel
         """
-        panel.Panel.__init__(self, parent)
+        panel.Panel.__init__(self, parent, name)
         self.collection = collection
 
         self.box = self.builder.get_object('files_box')
@@ -66,18 +83,7 @@ class FilesPanel(panel.Panel):
 
         self._setup_tree()
         self._setup_widgets()
-        self.menu = menu.FilesPanelMenu()
-        self.menu.connect('append-items', lambda *e:
-            self.emit('append-items', self.tree.get_selected_tracks()))
-        self.menu.connect('replace-items', lambda *e:
-            self.emit('replace-items', self.tree.get_selected_tracks()))
-        self.menu.connect('queue-items', lambda *e:
-            self.emit('queue-items', self.tree.get_selected_tracks()))
-        self.menu.connect('rating-changed', self.set_rating)
-        self.menu.connect('properties', lambda *e:
-            self.properties_dialog())
-        self.menu.connect('view-items', lambda *e:
-            self._on_view_items())
+        self.menu = menus.FilesContextMenu(self)
 
         self.key_id = None
         self.i = 0
@@ -87,35 +93,6 @@ class FilesPanel(panel.Panel):
         self.history = [first_dir]
         self.load_directory(first_dir, False)
 
-    def properties_dialog(self):
-        """
-            Shows the properties dialog
-        """
-        from xlgui import properties
-        tracks = self.tree.get_selected_tracks()
-
-        if not tracks:
-            return False
-
-        tracks_sorted = trax.sort_tracks(
-            ('artist', 'date', 'album', 'discnumber', 'tracknumber'),
-            tracks)
-
-        dialog = properties.TrackPropertiesDialog(self.parent,
-            tracks_sorted)
-
-    def _on_view_items(self, *args):
-        """
-            Opens a file manager in the containing directory
-        """
-        track = self.tree.get_selected_tracks()[0]
-        common.open_file_directory(track.get_loc_for_io())
-
-    def set_rating(self, widget, rating):
-        tracks = self.tree.get_selected_tracks()
-        for track in tracks:
-            track.set_rating(rating)
-
     def _setup_tree(self):
         """
             Sets up tree widget for the files panel
@@ -136,12 +113,10 @@ class FilesPanel(panel.Panel):
 
         pb = gtk.CellRendererPixbuf()
         text = gtk.CellRendererText()
-        # TRANSLATORS: Filename column in the file browser
         self.colname = colname = gtk.TreeViewColumn(_('Filename'))
         colname.pack_start(pb, False)
         colname.pack_start(text, True)
         if settings.get_option('gui/ellipsize_text_in_panels', False):
-            import pango
             text.set_property('ellipsize-set', True)
             text.set_property('ellipsize', pango.ELLIPSIZE_END)
         else:
@@ -175,8 +150,8 @@ class FilesPanel(panel.Panel):
         """
         self.directory = self.tree.render_icon(
             gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.track = gtk.gdk.pixbuf_new_from_file(
-            xdg.get_data_path('images/track.png'))
+        self.track = icons.MANAGER.pixbuf_from_icon_name(
+            'audio-x-generic', gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.back = self.builder.get_object('files_back_button')
         self.back.connect('clicked', self.go_back)
         self.forward = self.builder.get_object('files_forward_button')
@@ -198,11 +173,10 @@ class FilesPanel(panel.Panel):
         self.entry.connect('activate', self.entry_activate)
 
         # Set up the search entry
-        self.search = self.builder.get_object('files_search_entry')
-        self.search.connect('key-release-event', self.key_release)
-        self.search.connect('activate', lambda *e:
+        self.filter = guiutil.SearchEntry(self.builder.get_object('files_search_entry'))
+        self.filter.connect('activate', lambda *e:
             self.load_directory(self.current, history=False,
-            keyword=unicode(self.search.get_text(), 'utf-8')))
+                keyword=unicode(self.filter.get_text(), 'utf-8')))
 
     def fill_libraries_location(self, *e):
         model = self.location_bar.get_model()
@@ -248,17 +222,11 @@ class FilesPanel(panel.Panel):
             return True
 
         if event.keyval == gtk.keysyms.F5:
-            (mods,paths) = self.tree.get_selection().get_selected_rows()
             self.refresh(self.tree)
-            if paths and paths[0]:
-                try:
-                    self.tree.set_cursor(paths[0], None, False)
-                except:
-                    pass
             return True
         return False
 
-    def button_press(self, button, event):
+    def button_release(self, button, event):
         """
             Called when the user clicks on the playlist
         """
@@ -293,26 +261,14 @@ class FilesPanel(panel.Panel):
             if ftype == gio.FILE_TYPE_DIRECTORY:
                 self.load_directory(f)
             else:
-                self.emit('append-items', self.tree.get_selected_tracks())
-
-    def key_release(self, *e):
-        """
-            Called when someone releases a key.
-            Sets up a timer to simulate live-search
-        """
-        if self.key_id:
-            glib.source_remove(self.key_id)
-            self.key_id = None
-
-        self.key_id = glib.timeout_add(700, lambda *e:
-            self.load_directory(self.current, history=False,
-            keyword=unicode(self.search.get_text(), 'utf-8')))
+                self.emit('append-items', self.tree.get_selected_tracks(), True)
 
     def refresh(self, widget):
         """
             Refreshes the current view
         """
-        self.load_directory(self.current, False)
+        cursor = self.tree.get_cursor()
+        self.load_directory(self.current, False, cursor=cursor)
 
     def entry_activate(self, widget, event=None):
         """
@@ -324,12 +280,16 @@ class FilesPanel(panel.Panel):
         f = gio.file_parse_name(path)
         try:
             ftype = f.query_info('standard::type').get_file_type()
-        except glib.GError:
+        except glib.GError, e:
+            logger.error(e)
             self.entry.set_text(self.current.get_parse_name())
             return
         if ftype != gio.FILE_TYPE_DIRECTORY:
             f = f.get_parent()
         self.load_directory(f)
+        
+    def focus(self):
+        self.tree.grab_focus()
 
     def go_forward(self, widget):
         """
@@ -378,18 +338,24 @@ class FilesPanel(panel.Panel):
         settings.set_option(name, col.get_width())
 
     @common.threaded
-    def load_directory(self, directory, history=True, keyword=None):
+    def load_directory(self, directory, history=True, keyword=None, cursor=None):
         """
-            Loads a directory into the files view
+            Load a directory into the files view.
+
+            :param history: whether to record in history
+            :param keyword: filter string
+            :param cursor: path or (path, column) to select after loading.
+                    Useful while refreshing a directory.
         """
         self.current = directory
         try:
             infos = directory.enumerate_children('standard::is-hidden,'
                 'standard::name,standard::display-name,standard::type')
-        except gio.Error:
-            if directory.get_path() != xdg.homedir:
+        except gio.Error, e:
+            logger.error(e)
+            if directory.get_path() != xdg.homedir: # Avoid infinite recursion.
                 return self.load_directory(
-                    gio.File(xdg.homedir), history, keyword)
+                    gio.File(xdg.homedir), history, keyword, cursor)
         if self.current != directory: # Modified from another thread.
             return
 
@@ -397,25 +363,23 @@ class FilesPanel(panel.Panel):
 
         subdirs = []
         subfiles = []
-        import locale
         for info in infos:
             if info.get_is_hidden():
                 # Ignore hidden files. They can still be accessed manually from
                 # the location bar.
                 continue
-            f = directory.get_child(info.get_name())
-            basename = f.get_basename()
-            low_basename = basename.lower()
-            if keyword and keyword.lower() not in low_basename:
+            name = unicode(info.get_display_name(), 'utf-8')
+            low_name = name.lower()
+            if keyword and keyword.lower() not in low_name:
                 continue
+            f = directory.get_child(info.get_name())
             def sortkey():
-                name = info.get_display_name()
                 sortname = locale.strxfrm(name)
                 return sortname, name, f
             ftype = info.get_file_type()
             if ftype == gio.FILE_TYPE_DIRECTORY:
                 subdirs.append(sortkey())
-            elif any(low_basename.endswith('.' + ext)
+            elif any(low_name.endswith('.' + ext)
                     for ext in metadata.formats):
                 subfiles.append(sortkey())
 
@@ -426,17 +390,29 @@ class FilesPanel(panel.Panel):
             if self.current != directory: # Modified from another thread.
                 return
 
-            self.model.clear()
+            model = self.model
+            view = self.tree
 
+            model.clear()
             for sortname, name, f in subdirs:
-                self.model.append((f, self.directory, name, ''))
-
+                model.append((f, self.directory, name, ''))
             for sortname, name, f in subfiles:
-                size = f.query_info('standard::size').get_size() // 1024
-                size = locale.format_string(_("%d KB"), size, True)
-                self.model.append((f, self.track, name, size))
+                size = f.query_info('standard::size').get_size() // 1000
+                
+                # locale.format_string does not support unicode objects
+                # correctly, so we call it with an str and convert the 
+                # locale-dependent output to unicode.
+                size = locale.format_string('%d', size, True)
+                size = _('%s kB') % unicode(size, locale.getpreferredencoding())
+                
+                model.append((f, self.track, name, size))
 
-            self.tree.set_model(self.model)
+            if cursor:
+                view.set_cursor(cursor)
+            else:
+                view.set_cursor(0)
+                if view.flags() & gtk.REALIZED:
+                    view.scroll_to_point(0, 0)
 
             self.entry.set_text(directory.get_parse_name())
             if history:
@@ -468,14 +444,19 @@ class FilesPanel(panel.Panel):
         tracks = self.tree.get_selected_tracks()
         if not tracks: return
         for track in tracks:
-            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
+            DragTreeView.dragged_data[track.get_loc_for_io()] = track
         uris = trax.util.get_uris_from_tracks(tracks)
         selection.set_uris(uris)
 
-class FilesDragTreeView(guiutil.DragTreeView):
+class FilesDragTreeView(DragTreeView):
     """
         Custom DragTreeView to retrieve data from files
     """
+    
+    def get_selection_empty(self):
+        '''Returns True if there are no selected items'''
+        return self.get_selection().count_selected_rows() == 0
+    
     def get_selected_tracks(self):
         """
             Returns the currently selected tracks
@@ -488,7 +469,7 @@ class FilesDragTreeView(guiutil.DragTreeView):
             f = model[path][0]
             self.append_recursive(tracks, f)
 
-        return tracks or None
+        return tracks
 
     def append_recursive(self, songs, f):
         """
@@ -514,4 +495,3 @@ class FilesDragTreeView(guiutil.DragTreeView):
             return None
         tr = trax.Track(uri)
         return tr
-
