@@ -44,7 +44,6 @@ class EnginePreference(widgets.ComboPreference):
 class AudioSinkPreference(widgets.ComboPreference):
     default = "auto"
     name = 'player/audiosink'
-    restart_required = True
     
     def __init__(self, preferences, widget):
         widgets.ComboPreference.__init__(self, preferences, widget)
@@ -94,8 +93,6 @@ class SelectDeviceForSinkPreference(widgets.ComboPreference, widgets.Conditional
     name = "player/audiosink_device"
     condition_preference_name = 'player/audiosink'
 
-    restart_required = True
-
     def __init__(self, preferences, widget):
         widgets.ComboPreference.__init__(self, preferences, widget)
         widgets.Conditional.__init__(self)
@@ -123,18 +120,16 @@ class SelectDeviceForSinkPreference(widgets.ComboPreference, widgets.Conditional
         return False
 
     def on_condition_met(self):
-        self.widget.get_parent().show()
+        parent = self.widget.get_parent()
+        if parent is not None:
+            parent.show()
 
     def on_condition_failed(self):
         self.widget.get_parent().hide()
         self.widget.get_model().clear()
-
-    def _setup_changed(self):
-        self.widget.connect('changed', self._change)
-
-    def _change(self, *args):
-        if self.is_enabled:
-            self.change(args)
+        
+    def done(self):
+        return self.is_enabled
 
     def _get_value(self):
         if self.is_enabled:
@@ -157,15 +152,16 @@ class RemoveQueuedItemWhenPlayed(widgets.CheckPreference):
     default = True
     name = 'queue/remove_item_when_played'
     
-class AutoAdvancePlayer(widgets.CheckPreference):
-    default = True
-    name = 'player/auto_advance'
+class DisableNewTrackWhenPlaying(widgets.CheckPreference):
+    default = False
+    name = 'queue/disable_new_track_when_playing'
 
-class UnifiedConditional(widgets.Conditional):
+class EngineConditional(widgets.Conditional):
     """
-        True if the unified engine is selected
+        True if the specified engine is selected
     """
     condition_preference_name = 'player/engine'
+    conditional_engine = ''
 
     def on_check_condition(self):
         """
@@ -177,45 +173,75 @@ class UnifiedConditional(widgets.Conditional):
         iter = self.condition_widget.get_active_iter()
         value = self.condition_widget.get_model().get_value(iter, 0)
 
-        if value == 'unified':
+        if value == self.conditional_engine:
+            return True
+
+        return False
+    
+class AutoAdvancePlayer(widgets.CheckPreference, EngineConditional):
+    default = True
+    name = 'player/auto_advance'
+    conditional_engine = 'normal'
+
+    def __init__(self, preferences, widget):
+        widgets.CheckPreference.__init__(self, preferences, widget)
+        EngineConditional.__init__(self)
+    
+class AutoAdvanceDelay(widgets.SpinPreference, widgets.MultiConditional):
+    default = 0
+    name = "player/auto_advance_delay"
+    condition_preference_names = ['player/auto_advance', 'player/engine']
+    
+    def __init__(self, preferences, widget):
+        widgets.SpinPreference.__init__(self, preferences, widget)
+        widgets.MultiConditional.__init__(self)
+        
+    def on_check_condition(self):
+        if not self.condition_widgets['player/auto_advance'].get_active():
+            return False
+            
+        iter = self.condition_widgets['player/engine'].get_active_iter()
+        value = self.condition_widgets['player/engine'].get_model().get_value(iter, 0)
+
+        if value == 'normal':
             return True
 
         return False
 
-class UserFadeTogglePreference(widgets.CheckPreference, UnifiedConditional):
+class UserFadeTogglePreference(widgets.CheckPreference, EngineConditional):
     default = False
     name = 'player/user_fade_enabled'
-    condition_preference_name = 'player/engine'
+    conditional_engine = 'unified'
 
     def __init__(self, preferences, widget):
         widgets.CheckPreference.__init__(self, preferences, widget)
-        UnifiedConditional.__init__(self)
+        EngineConditional.__init__(self)
 
-class UserFadeDurationPreference(widgets.SpinPreference, UnifiedConditional):
+class UserFadeDurationPreference(widgets.SpinPreference, EngineConditional):
     default = 1000
     name = 'player/user_fade'
-    condition_preference_name = 'player/engine'
+    conditional_engine = 'unified'
 
     def __init__(self, preferences, widget):
         widgets.SpinPreference.__init__(self, preferences, widget)
-        UnifiedConditional.__init__(self)
+        EngineConditional.__init__(self)
 
-class CrossfadingPreference(widgets.CheckPreference, UnifiedConditional):
+class CrossfadingPreference(widgets.CheckPreference, EngineConditional):
     default = False
     name = 'player/crossfading'
-    condition_preference_name = 'player/engine'
+    conditional_engine = 'unified'
 
     def __init__(self, preferences, widget):
         widgets.CheckPreference.__init__(self, preferences, widget)
-        UnifiedConditional.__init__(self)
+        EngineConditional.__init__(self)
 
-class CrossfadeDurationPreference(widgets.SpinPreference, UnifiedConditional):
+class CrossfadeDurationPreference(widgets.SpinPreference, EngineConditional):
     default = 1000
     name = 'player/crossfade_duration'
-    condition_preference_name = 'player/engine'
+    conditional_engine = 'unified'
 
     def __init__(self, preferences, widget):
         widgets.SpinPreference.__init__(self, preferences, widget)
-        UnifiedConditional.__init__(self)
+        EngineConditional.__init__(self)
 
 # vim: et sts=4 sw=4
