@@ -33,6 +33,7 @@ import os
 import pango
 import random
 import re
+import sys
 
 from xl.nls import gettext as _
 from xl.playlist import (
@@ -151,18 +152,18 @@ class RemoveCurrentMenuItem(menu.MenuItem):
         """
         item = gtk.ImageMenuItem(_('Remove Current Track From Playlist'))
         item.set_image(gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU))
-        item.connect('activate', self.on_activate)
+        item.connect('activate', self.on_activate, parent, context)
 
         if player.PLAYER.is_stopped():
             item.set_sensitive(False)
 
         return item
 
-    def on_activate(self, menuitem, playlist):
+    def on_activate(self, menuitem, parent, context):
         """
             Removes the currently playing track from the current playlist
         """
-        playlist = self.get_playlist_func()
+        playlist = self.get_playlist_func(parent, context)
         
         if playlist and playlist.current == player.PLAYER.current:
             del playlist[playlist.current_position]
@@ -181,7 +182,7 @@ class RandomizeMenuItem(menu.MenuItem):
         """
         label = _('Randomize Playlist')
 
-        if context['selection-count'] > 1:
+        if not context['selection-empty']:
             label = _('Randomize Selection')
 
         item = gtk.MenuItem(label)
@@ -215,7 +216,7 @@ def __create_playlist_tab_context_menu():
         playlists = main.exaile().playlists
         name = dialogs.ask_for_playlist_name(playlists, page.playlist.name)
         if name is not None:
-            page.set_name(name)
+            page.set_page_name(name)
             playlists.save_playlist(page.playlist)
     
     items.append(smi('save', ['new-tab-sep'], None, 'gtk-save',
@@ -228,7 +229,14 @@ def __create_playlist_tab_context_menu():
     items.append(smi('clear', ['rename'], None, 'gtk-clear',
         lambda w, n, o, c: o.playlist.clear()))
     items.append(sep('tab-close-sep', ['clear']))
-    items.append(smi('tab-close', ['tab-close-sep'], None, 'gtk-close',
+    
+    def _get_pl_func(o, c):
+        return o.playlist
+    
+    items.append(menuitems.ExportPlaylistMenuItem('export', ['tab-close-sep'], _get_pl_func))
+    items.append(menuitems.ExportPlaylistFilesMenuItem('export-files', ['export'], _get_pl_func))
+    items.append(sep('tab-export-sep', ['export']))
+    items.append(smi('tab-close', ['tab-export-sep'], None, 'gtk-close',
         lambda w, n, o, c: o.tab.close()))
     for item in items:
         providers.register('playlist-tab-context-menu', item)
@@ -245,7 +253,7 @@ class PlaylistContextMenu(menu.ProviderMenu):
     def get_context(self):
         context = common.LazyDict(self._parent)
         context['playlist'] = lambda name, parent: parent.playlist
-        context['selection-count'] = lambda name, parent: parent.get_selection_count()
+        context['selection-empty'] = lambda name, parent: parent.get_selection_count() == 0
         context['selected-paths'] = lambda name, parent: parent.get_selected_paths()
         context['selected-items'] = lambda name, parent: parent.get_selected_items()
         context['selected-tracks'] = lambda name, parent: parent.get_selected_tracks()
@@ -420,10 +428,10 @@ class PlaylistPage(NotebookPage):
     def focus(self):
         self.view.grab_focus()
 
-    def get_name(self):
+    def get_page_name(self):
         return self.playlist.name
 
-    def set_name(self, name):
+    def set_page_name(self, name):
         self.playlist.name = name
         self.name_changed()
 
@@ -591,10 +599,17 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
         self.menu = PlaylistContextMenu(self)
         self.tabmenu = menu.ProviderMenu('playlist-tab-context-menu', self)
         self.dragging = False
+        self.pending_event = None
         self.button_pressed = False # used by columns to determine whether
                                     # a notify::width event was initiated
                                     # by the user.
         self._insert_focusing = False
+        
+        self._hack_is_osx = sys.platform == 'darwin'
+        self._hack_osx_control_mask = False
+
+        # Set to true if you only want things to be copied here, not moved
+        self.dragdrop_copyonly = False
 
         self.set_fixed_height_mode(True) # MASSIVE speedup - don't disable this!
         self.set_rules_hint(True)
@@ -669,22 +684,36 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
 
     def get_selected_paths(self):
         """
-            Returns a list of pairs of treepaths
-            which are currently selected in the playlist.
+            Returns a list of pairs of treepaths which are currently
+            selected in the playlist.
+            
+            The treepaths are returned for the base model, so they are
+            indices that can be used with the playlist currently
+            associated with this view.
         """
         selection = self.get_selection()
         model, paths = selection.get_selected_rows()
+        
+        if isinstance(model, gtk.TreeModelFilter):
+            paths = [model.convert_path_to_child_path(path) for path in paths]
+        
         return paths
 
     def get_selected_items(self):
         """
             Returns a list of pairs of indices and :class:`xl.trax.Track`
             which are currently selected in the playlist.
+            
+            The indices can be used with the playlist currently associated
+            with this view.
         """
-        paths = self.get_selected_paths()
-        model = self.get_model()
+        selection = self.get_selection()
+        model, paths = selection.get_selected_rows()
         try:
-            tracks = [(path[0], model.get_value(model.get_iter(path), 0)) for path in paths]
+            if isinstance(model, gtk.TreeModelFilter):
+                tracks = [(model.convert_path_to_child_path(path)[0], model.get_value(model.get_iter(path), 0)) for path in paths]
+            else:
+                tracks = [(path[0], model.get_value(model.get_iter(path), 0)) for path in paths]
         except TypeError: #one of the paths was invalid
             return []
         return tracks
@@ -715,10 +744,11 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
     def _play_track_at(self, position, track, on_activated=False):
         '''Internal API'''       
         if not settings.get_option('playlist/enqueue_by_default', False) or \
-                (self.playlist is self.player.queue and on_activated):        
-            self.playlist.set_current_position(position)
-            self.player.queue.set_current_playlist(self.playlist)
-            self.player.queue.play(track=track)
+                (self.playlist is self.player.queue and on_activated):
+            if self.player.queue.is_play_enabled():        
+                self.playlist.set_current_position(position)
+                self.player.queue.set_current_playlist(self.playlist)
+                self.player.queue.play(track=track)
         elif self.playlist is not self.player.queue:
             self.player.queue.append(track)
             
@@ -837,7 +867,7 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
         
     def on_cursor_changed(self, widget):
         context = common.LazyDict(self)
-        context['selection-count'] = lambda name, parent: parent.get_selection_count()
+        context['selection-empty'] = lambda name, parent: parent.get_selection_count() == 0
         context['selected-items'] = lambda name, parent: parent.get_selected_items()
         context['selected-tracks'] = lambda name, parent: parent.get_selected_tracks()
         event.log_event( 'playlist_cursor_changed', self, context)
@@ -880,11 +910,20 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
         """
         self.button_pressed = True
         self.grab_focus()
+        
+        # need this to workaround bug in GTK+ on OSX when dragging/dropping
+        # -> https://bugzilla.gnome.org/show_bug.cgi?id=722815
+        if self._hack_is_osx:
+            self._hack_osx_control_mask = True if e.state & gtk.gdk.CONTROL_MASK else False
 
         selection = self.get_selection()
-        path = self.get_path_at_pos(int(e.x), int(e.y))
+        pathtuple = self.get_path_at_pos(int(e.x), int(e.y))
         # We only need the tree path if present
-        path = path[0] if path else None
+        if pathtuple:
+            path = pathtuple[0]
+            col = pathtuple[1]
+        else:
+            path = None
 
         # We unselect all selected items if the user clicks on an empty
         # area of the treeview and no modifier key is active
@@ -897,6 +936,7 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
             if e.button == 1 and not e.state & gtk.accelerator_get_default_mod_mask() and \
                selection.path_is_selected(path):
                 selection.set_select_function(lambda *args: False)
+                self.pending_event = (path, col)
 
             # Open the context menu on right clicks
             if e.button == 3:
@@ -919,9 +959,17 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
             Unsets the internal state for button press
         """
         self.button_pressed = False
+        self._hack_osx_control_mask = False
+        
         # Restore regular selection behavior in any case
         self.get_selection().set_select_function(lambda *args: True)
-
+        
+        if self.pending_event:
+            path, col = self.pending_event
+            # perform the normal selection that would have happened
+            self.set_cursor(path, col, 0)
+            self.pending_event = None
+        
         return gtk.TreeView.do_button_release_event(self, e)
 
     def on_key_press_event(self, widget, event):
@@ -952,6 +1000,7 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
         """
         # TODO: set drag icon
         self.dragging = True
+        self.pending_event = None
 
     def on_drag_data_get(self, widget, context, selection, info, etime):
         """
@@ -999,6 +1048,10 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
 
         if drop_info:
             path, position = drop_info
+            model = self.get_model()
+            if isinstance(model, gtk.TreeModelFilter):
+                path = model.convert_path_to_child_path(path)
+            
             insert_position = path[0]
             if position in (gtk.TREE_VIEW_DROP_AFTER, gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
                 insert_position += 1
@@ -1048,7 +1101,8 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
                 else:
                     tracks.extend(trax.get_tracks_from_uri(uri))
             sort_by, reverse = self.get_sort_by()
-            tracks = trax.sort_tracks(sort_by, tracks, reverse=reverse)
+            tracks = trax.sort_tracks(sort_by, tracks, reverse=reverse,
+                artist_compilations=True)
             if insert_position >= 0:
                 self.playlist[insert_position:insert_position] = tracks
             else:
@@ -1087,38 +1141,41 @@ class PlaylistView(AutoScrollTreeView, providers.ProviderHandler):
         x, y, modifier = self.window.get_pointer()
         target = self.drag_dest_find_target(context, self.drag_dest_get_target_list())
 
-        if modifier & gtk.gdk.CONTROL_MASK or target == 'text/uri-list':
+        if target == 'text/uri-list' or \
+           (self._hack_is_osx and self._hack_osx_control_mask) or \
+           (not self._hack_is_osx and modifier & gtk.gdk.CONTROL_MASK):
             action = gtk.gdk.ACTION_COPY
-
+        
+        if self.dragdrop_copyonly and context.get_source_widget() != self:
+            action = gtk.gdk.ACTION_COPY
+        
         context.drag_status(action, etime)
 
         return True
 
     def show_properties_dialog(self):
         from xlgui import properties
-        tracks = self.get_selected_tracks()
-        current_position = 0
+        items = self.get_selected_items()
+        
         # If only one track is selected, we expand `tracks` to include all
-        # tracks in the playlist.
-        if len(tracks) == 1:
+        # tracks in the playlist... except for really large playlists, this
+        # essentially hangs. Instead, only show all tracks *if* the playlist
+        # size is less than 100.
+        # 
+        # A better option would be to lazy load the files, but I'm too lazy
+        # to implement that now.. :)  
+        
+        if len(items) == 1 and len(self.playlist) < 100:
             tracks = self.playlist[:]
-
-            model = self.get_model()
-            path, column = self.get_cursor()
-
-            # If the model has been placed within a filter model,
-            # convert the path to the path of the real model
-            if isinstance(model, gtk.TreeModelFilter):
-                path = model.convert_path_to_child_path(path)
-
-            current_position = path[0]
+            current_position = items[0][0]
             with_extras = True
         else:
+            tracks = [i[1] for i in items]
+            current_position = 0
             with_extras = False
-
-        if tracks:
-            dialog = properties.TrackPropertiesDialog(None,
-                    tracks, current_position, with_extras)
+        
+        properties.TrackPropertiesDialog(None, tracks,
+                                         current_position, with_extras)
 
     def on_provider_removed(self, provider):
         """
