@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import urlparse
 import urllib
@@ -21,9 +21,10 @@ import logging
 import dbus
 import gtk
 import glib
+import os
 import tempfile
 
-from xl import covers
+from xl import covers, player
 import xl.event
 import xl.settings
 
@@ -55,6 +56,7 @@ class ExaileAwn(object):
         self.exaile = None
         self.enabled = True
         self.timer_id = None
+        self.temp_icon_path = None
 
     def enable_progress(self, type, player, object):
         assert self.timer_id is None
@@ -103,19 +105,24 @@ class ExaileAwn(object):
         elif not hasattr(self.exaile, 'player'):
             log.debug("Player not loaded, ignoring set_cover call")
             return
-        elif self.exaile.player.current is None:
+
+        try:
+            os.remove(self.temp_icon_path)
+        except (TypeError, OSError):
+            pass
+
+        if player.PLAYER.current is None:
             log.debug("Player stopped, removing AWN cover")
             self.unset_cover()
         elif not self.cover_display:
             self.unset_cover()
         else:
-            image_data = covers.MANAGER.get_cover(self.exaile.player.current)
-
-            if image_data is not None:
-                pixbuf = icons.MANAGER.pixbuf_from_data(image_data)
-                descriptor, path = tempfile.mkstemp()
-                pixbuf.save(path, 'png')
-                self.awn.SetTaskIconByXid(self.xid(), path)
+            image_data = covers.MANAGER.get_cover(player.PLAYER.current,
+                set_only=True, use_default=True)
+            pixbuf = icons.MANAGER.pixbuf_from_data(image_data)
+            descriptor, self.temp_icon_path = tempfile.mkstemp()
+            pixbuf.save(self.temp_icon_path, 'png')
+            self.awn.SetTaskIconByXid(self.xid(), self.temp_icon_path)
 
     def unset_timer(self):
         self._set_timer(100)
@@ -138,9 +145,9 @@ class ExaileAwn(object):
     def update_timer(self, *args, **kwargs):
         if self.exaile is None:
             return False
-        if self.exaile.player is None:
+        if player.PLAYER is None:
             return False
-        track = self.exaile.player.current
+        track = player.PLAYER.current
         # Not playing anything
         if track is None:
             return False
@@ -148,7 +155,7 @@ class ExaileAwn(object):
         if not track.is_local() and not track.get_tag_raw('__length'):
             self._set_timer(100)
             return False
-        self._set_timer(int(self.exaile.player.get_progress() * 100))
+        self._set_timer(int(player.PLAYER.get_progress() * 100))
         return True
 
     def on_option_set(self, event, settings, option):
@@ -175,13 +182,13 @@ def enable(exaile):
         EXAILE_AWN = ExaileAwn()
     EXAILE_AWN.exaile = exaile
     for signal in TRACK_CHANGE_CALLBACKS:
-        xl.event.add_callback(EXAILE_AWN.set_cover, signal)
+        xl.event.add_callback(EXAILE_AWN.set_cover, signal, player.PLAYER)
     xl.event.add_callback(EXAILE_AWN.enable_progress,
-            'playback_player_start')
+            'playback_player_start', player.PLAYER)
     xl.event.add_callback(EXAILE_AWN.disable_progress,
-            'playback_player_end')
+            'playback_player_end', player.PLAYER)
     xl.event.add_callback(EXAILE_AWN.toggle_pause_progress,
-            'playback_toggle_pause')
+            'playback_toggle_pause', player.PLAYER)
     xl.event.add_callback(EXAILE_AWN.on_option_set, 'plugin_awn_option_set')
     EXAILE_AWN.set_cover()
 
@@ -190,11 +197,11 @@ def disable(exaile):
     for signal in TRACK_CHANGE_CALLBACKS:
         xl.event.remove_callback(EXAILE_AWN.set_cover, signal)
     xl.event.remove_callback(EXAILE_AWN.enable_progress,
-            'playback_player_start')
+            'playback_player_start', player.PLAYER)
     xl.event.remove_callback(EXAILE_AWN.disable_progress,
-            'playback_player_end')
+            'playback_player_end', player.PLAYER)
     xl.event.remove_callback(EXAILE_AWN.toggle_pause_progress,
-            'playback_toggle_pause')
+            'playback_toggle_pause', player.PLAYER)
     EXAILE_AWN.unset_cover()
     EXAILE_AWN.unset_timer()
     EXAILE_AWN.exaile = None
