@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Aren Olson
+# Copyright (C) 2009-2010 Aren Olson
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -27,13 +27,11 @@
 
 from xl.nls import gettext as _
 from xl import providers, event
-from xl.hal import Handler
-from xl.devices import Device
+from xl.hal import Handler, UDisksProvider
+from xl.devices import Device, KeyedDevice
 import logging
 logger = logging.getLogger(__name__)
 
-PROVIDER = None
-
 import dbus, threading, os, struct
 from fcntl import ioctl
 from xl import playlist, trax, common
@@ -48,7 +46,7 @@ except:
 
 import cdprefs
 
-def get_prefs_pane():
+def get_preferences_pane():
     return cdprefs
 
 TOC_HEADER_FMT = 'BB'
@@ -60,16 +58,41 @@ CDROM_LEADOUT = 0xAA
 CDROM_MSF = 0x02
 CDROM_DATA_TRACK = 0x04
 
-def enable(exaile):
-    global PROVIDER
-    PROVIDER = CDHandler()
-    providers.register("hal", PROVIDER)
+class CdPlugin(object):
+    
+    def enable(self, exaile):
+        self.exaile = exaile
+        self.hal = None
+        self.udisks = None
+        self.udisks2 = None
+    
+    def on_exaile_loaded(self):
+        
+        # verify that hal/whatever is loaded, load correct provider
+        
+        if self.exaile.udisks2 is not None:
+            self.udisks2 = UDisks2CdProvider()
+            providers.register('udisks2', self.udisks2)
+            
+        elif self.exaile.udisks is not None:
+            self.udisks = UDisksCdProvider()
+            providers.register('udisks', self.udisks)
+            
+        elif self.exaile.hal is not None:
+            self.udisks = HALCdProvider()
+            providers.register('hal', self.udisks)
+        
+    
+    def disable(self, exaile):
+        if self.hal is not None:
+            providers.unregister('hal', self.hal)
+        if self.udisks is not None:
+            providers.unregister('udisks', self.udisks)
+        if self.udisks2 is not None:
+            providers.unregister('udisks2', self.udisks2)
 
+plugin_class = CdPlugin
 
-def disable(exaile):
-    global PROVIDER
-    providers.unregister("hal", PROVIDER)
-    PROVIDER = None
 
 class CDTocParser(object):
     #based on code from http://carey.geek.nz/code/python-cdrom/cdtoc.py
@@ -86,26 +109,29 @@ class CDTocParser(object):
 
     def read_toc(self):
         fd = os.open(self.device, os.O_RDONLY)
-        toc_header = struct.pack(TOC_HEADER_FMT, 0, 0)
-        toc_header = ioctl(fd, CDROMREADTOCHDR, toc_header)
-        start, end = struct.unpack(TOC_HEADER_FMT, toc_header)
-
-        self.raw_tracks = []
-
-        for trnum in range(start, end + 1) + [CDROM_LEADOUT]:
-            entry = struct.pack(TOC_ENTRY_FMT, trnum, 0, CDROM_MSF, 0)
-            entry = ioctl(fd, CDROMREADTOCENTRY, entry)
-            track, adrctrl, format, addr = struct.unpack(TOC_ENTRY_FMT, entry)
-            m, s, f = struct.unpack(ADDR_FMT, struct.pack('i', addr))
-
-            adr = adrctrl & 0xf
-            ctrl = (adrctrl & 0xf0) >> 4
-
-            data = 0
-            if ctrl & CDROM_DATA_TRACK:
-                data = 1
-
-            self.raw_tracks.append( (track, m, s, f, (m*60+s) * 75 + f, data) )
+        try:
+            toc_header = struct.pack(TOC_HEADER_FMT, 0, 0)
+            toc_header = ioctl(fd, CDROMREADTOCHDR, toc_header)
+            start, end = struct.unpack(TOC_HEADER_FMT, toc_header)
+    
+            self.raw_tracks = []
+    
+            for trnum in range(start, end + 1) + [CDROM_LEADOUT]:
+                entry = struct.pack(TOC_ENTRY_FMT, trnum, 0, CDROM_MSF, 0)
+                entry = ioctl(fd, CDROMREADTOCENTRY, entry)
+                track, adrctrl, format, addr = struct.unpack(TOC_ENTRY_FMT, entry)
+                m, s, f = struct.unpack(ADDR_FMT, struct.pack('i', addr))
+    
+                adr = adrctrl & 0xf
+                ctrl = (adrctrl & 0xf0) >> 4
+    
+                data = 0
+                if ctrl & CDROM_DATA_TRACK:
+                    data = 1
+    
+                self.raw_tracks.append( (track, m, s, f, (m*60+s) * 75 + f, data) )
+        finally:
+            os.close(fd)
 
     def get_raw_info(self):
         return self.raw_tracks[:]
@@ -138,9 +164,9 @@ class CDPlaylist(playlist.Playlist):
 
         for count, length in enumerate(lengths):
             count += 1
-            song = trax.Track("cdda://%d#%s" % (count, self.device))
+            song = trax.Track("cdda://%d/#%s" % (count, self.device))
             song.set_tag_raw('title', "Track %d" % count)
-            song.set_tag_raw('tracknumber', count)
+            song.set_tag_raw('tracknumber', str(count))
             song.set_tag_raw('__length', length)
             songs[song.get_loc_for_io()] = song
 
@@ -150,7 +176,7 @@ class CDPlaylist(playlist.Playlist):
         sort_tups.sort()
         sorted = [ s[1] for s in sort_tups ]
 
-        self.add_tracks(sorted)
+        self.extend(sorted)
 
         if CDDB_AVAIL:
             self.get_cddb_info()
@@ -174,7 +200,7 @@ class CDPlaylist(playlist.Playlist):
 
         title = info['DTITLE'].split(" / ")
         for i in range(self.info[1]):
-            tr = self.ordered_tracks[i]
+            tr = self[i]
             tr.set_tag_raw('title',
                     info['TTITLE' + `i`].decode('iso-8859-15', 'replace'))
             tr.set_tag_raw('album',
@@ -186,16 +212,16 @@ class CDPlaylist(playlist.Playlist):
             tr.set_tag_raw('genre',
                     info['DGENRE'])
 
-        self.set_name(title[1].decode('iso-8859-15', 'replace'))
+        self.name = title[1].decode('iso-8859-15', 'replace')
         event.log_event('cddb_info_retrieved', self, True)
 
-class CDDevice(Device):
+class CDDevice(KeyedDevice):
     """
         represents a CD
     """
     class_autoconnect = True
 
-    def __init__(self, dev="/dev/cdrom"):
+    def __init__(self, dev):
         Device.__init__(self, dev)
         self.name = _("Audio Disc")
         self.dev = dev
@@ -220,8 +246,9 @@ class CDDevice(Device):
     def disconnect(self):
         self.playlists = []
         self.connected = False
+        CDDevice.destroy(self)
 
-class CDHandler(Handler):
+class HALCdProvider(Handler):
     name = "cd"
     def is_type(self, device, capabilities):
         if "volume.disc" in capabilities:
@@ -244,8 +271,55 @@ class CDHandler(Handler):
 
         return cddev
 
+class UDisksCdProvider(UDisksProvider):
+    name = 'cd'
+    PRIORITY = UDisksProvider.NORMAL
+
+    def get_priority(self, obj, udisks):
+        # DeviceChanged is called before and after tracks are read. We only want
+        # the second case, so use number of audio tracks to identify supported
+        # media. As a bonus, this means we never have to care about the type of
+        # disc (CD, DVD, etc.).
+        ntracks = obj.props.Get('OpticalDiscNumAudioTracks')
+        return self.PRIORITY if ntracks > 0 else None
+
+    def get_device(self, obj, udisks):
+        return CDDevice(str(obj.props.Get('DeviceFile')))
+    
+    def on_device_changed(self, obj, udisks, device):
+        if obj.props.Get('OpticalDiscNumAudioTracks') <= 0:
+            return 'remove'
+
+
+class UDisks2CdProvider(UDisksProvider):
+    name = 'cd'
+    PRIORITY = UDisksProvider.NORMAL
+    
+    def _get_num_tracks(self, obj, udisks):
+        if obj.iface_type != 'org.freedesktop.UDisks2.Block':
+            return
+        
+        try:
+            drive = udisks.get_object_by_path(obj.props.Get('Drive'))
+        except KeyError:
+            return
+        
+        # Use number of audio tracks to identify supported media
+        ntracks = drive.props.Get('OpticalNumAudioTracks')
+        if ntracks > 0:
+            return ntracks
+    
+    def get_priority(self, obj, udisks):
+        ntracks = self._get_num_tracks(obj, udisks)
+        if ntracks is not None:
+            return self.PRIORITY
+
+    def get_device(self, obj, udisks):
+        device = obj.props.Get('Device', byte_arrays=True).strip('\0')
+        return CDDevice(str(device))
+    
+    def on_device_changed(self, obj, udisks, device):
+        if self._get_num_tracks(obj, udisks) is None:
+            return 'remove'
 
 # vim: et sts=4 sw=4
-
-
-
