@@ -1,5 +1,5 @@
 # Copyright (C) 2006-2007 Aren Olson
-#                    2009 Brian Parma
+#                    2011 Brian Parma
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,8 +13,9 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+import traceback
 import os
 import gtk
 import dbus
@@ -24,16 +25,29 @@ import time
 import threading
 import gobject
 import xlgui
+import pickle
 from gettext import gettext as _
-from xl import collection, event, trax, common
 from xlgui.panel.collection import CollectionPanel
 from xlgui import guiutil
-from xlgui.widgets import dialogs
+from xlgui.widgets import dialogs, menu, menuitems
 from daap import DAAPClient, DAAPError
+from xl import (
+    collection, 
+    event, 
+    trax, 
+    common, 
+    providers, 
+    settings, 
+    xdg
+)
 
 logger = logging.getLogger(__name__)
 gobject.threads_init()
 
+_smi = menu.simple_menu_item
+_sep = menu.simple_separator
+
+
 #
 #   Check For python-avahi, we can work without
 #  avahi, but wont be able to discover shares.
@@ -42,8 +56,8 @@ gobject.threads_init()
 try:
     import avahi
     AVAHI = True
-except Exception, inst:
-    logger.warn('AVAHI exception: %s' % inst)
+except ImportError:
+    logger.warning('avahi not installed, can\'t auto-discover servers')
     AVAHI = False
 
 # detect authoriztion support in python-daap
@@ -56,84 +70,161 @@ except TypeError:
 except:
     AUTH = True
 
-#PLUGIN_ICON = gtk.Button().render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_MENU)
-
 # Globals Warming
-MENU_ITEM = None
 MANAGER = None
 
+class AttrDict(dict):
+    def __getattr__(self, name):
+        return self[name]
+
+import functools
+# helper function to parse avahi info into a list of tuples (for dict())
+parse = functools.partial(zip,
+        ['interface',
+        'protocol',
+        'name',
+        'type',
+        'domain',
+        'host',
+        'aprotocol',
+        'address',
+        'port',
+        'txt',
+        'flags'])
+
 class DaapAvahiInterface(gobject.GObject): #derived from python-daap/examples
     """
-        Handles detection of DAAP shares via Avahi and manages the menu showing the shares.
-    Fires a "connect" signal when a menu item is clicked.
+        Handles detection of DAAP shares via Avahi and manages the menu 
+        showing the shares.
+        
+        Fires a "connect" signal when a menu item is clicked.
     """
     __gsignals__ = {
                     'connect' : ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                                     ( gobject.TYPE_PYOBJECT, ) ) }
 
     def new_service(self, interface, protocol, name, type, domain, flags):
-        interface, protocol, name, type, domain, host, aprotocol, address, port, txt, flags = self.server.ResolveService(interface, protocol, name, type, domain, avahi.PROTO_UNSPEC, dbus.UInt32(0))
         """
             Called when a new share is found.
         """
-        logger.info("DAAP: Found %s." % name)
+        x = self.server.ResolveService(interface, protocol, name, type, domain, 
+                                        avahi.PROTO_UNSPEC, dbus.UInt32(0))
+        
+        x = AttrDict(parse(x))
+
+        logger.info("DAAP share found: '{0}' at ({1},{2})."
+                        .format(x.name, x.address, x.port))
+                        
+        # gstreamer can't handle link-local ipv6
+        if 'fe80' in x.address:
+            return
+
         #Use all available info in key to avoid name conflicts.
         nstr = '%s%s%s%s%s' % (interface, protocol, name, type, domain)
 
-        if name in self.services:   # using only name for conflicts (for multiple adapters)
+        if nstr in self.services:   
             return
+            
 
-        self.services[name] = (address, port)
-        self.new_share_menu_item(name)
+        self.services[nstr] = x
+        self.rebuild_share_menu_items()
+#        self.new_share_menu_item(x)
 
     def remove_service(self, interface, protocol, name, type, domain, flags):
         """
             Called when the connection to a share is lost.
         """
-        logger.info("DAAP: Lost %s." % name)
+        logger.info("DAAP share lost: %s." % name)
         nstr = '%s%s%s%s%s' % (interface, protocol, name, type, domain)
 
-        if name in self.services:
-            self.remove_share_menu_item(name)
-            del self.services[name]
+        if nstr in self.services:
+#            self.remove_share_menu_item(name)
+            del self.services[nstr]
+            self.rebuild_share_menu_items()
 
-    def new_share_menu_item(self, name):
+    def new_share_menu_item(self, name, key):
         '''
-            This function is called by Avahi when a new share is detected
-        on the network.  It adds the share to the Connect menu.
+            This function is called to add a server to the connect menu.
         '''
-
+        
+        # check if the menu exist and check if it's ipv4 or we are allowing
+        # ipv6
+        print 'adding menu',name,key
         if self.menu:
-            menu_item = gtk.MenuItem(name)
-            menu_item.connect('activate', self.clicked, name )
-            self.menu.append_item(menu_item)
-            menu_item.show()
+                menu_item = _smi(name, ['sep'], name,
+                                callback=lambda *x: self.clicked(key))
+                self.menu.add_item(menu_item)
 
     def remove_share_menu_item(self, name):
         '''
-            This function is called by Avahi when a share is removed
-            from the network.  It removes the menu entry for the share.
+            This function is called to remove a server from the connect menu.
         '''
 
         if self.menu:
-            item = [x for x in self.menu.get_children() if (x.get_name() == 'GtkMenuItem') and (unicode(x.get_child().get_text(), 'utf-8') == name)]
-            if len(item) > 0:
-                item[0].destroy()
+            for item in self.menu._items:
+                if item.name == name:
+                    self.menu.remove_item(item)
+                    break
 
-    def clicked(self, menu_item, name):
+    def clear_share_menu_items(self):
+        '''
+            This function is used to clear all the menu items out of a menu.
+        '''
+        if self.menu:
+            for item in self.menu._items:
+                if item.name == 'manual' or item.name == 'sep':
+                    continue
+                self.menu.remove_item(item)
+    
+    def rebuild_share_menu_items(self):
+        '''
+            This function fills the menu with known servers.
+        '''
+        self.clear_share_menu_items()
+        
+        show_ipv6 = settings.get_option('plugin/daapclient/ipv6', False)
+        items = {}
+        
+        for key,x in self.services.items():
+            name = '{0} ({1})'.format(x.name,x.host)
+            if x.protocol == avahi.PROTO_INET6:
+                if not show_ipv6:
+                    continue
+                name += ' - ipv6'
+
+            if name not in items:
+                items[name] = (key,x)
+        
+# this dedups based on name-host, replacing ipv4 with ipv6
+#        for key,x in self.services.items():
+#            name = '{0} ({1})'.format(x.name,x.host)
+#            if x.protocol == avahi.PROTO_INET6 and show_ipv6:
+#                if name in items:
+#                    # prefer ipv6
+#                    if items[name][1].protocol == avahi.PROTO_INET:
+#                        items[name] = (key,x)
+#            elif x.protocol == avahi.PROTO_INET:
+#                if name not in items:
+#                    items[name] = (key,x)
+        
+        for name in items:
+            self.new_share_menu_item(name, key=items[name][0])
+
+    def clicked(self, key):
         '''
             This function is called in response to a menu_item click.
         Fire away.
         '''
-        gobject.idle_add(self.emit, "connect", (name,)+self.services[name])
+        x = self.services[key]
+        gobject.idle_add(self.emit, "connect", (x.name, x.address, x.port, x))
 
-    def __init__(self, exaile, menu):
+    def __init__(self, exaile, _menu):
         """
             Sets up the avahi listener.
         """
         gobject.GObject.__init__(self)
         self.services = {}
-        self.menu = menu
+        self.menu = _menu
         self.bus = dbus.SystemBus()
         self.server = dbus.Interface(self.bus.get_object(avahi.DBUS_NAME,
             avahi.DBUS_PATH_SERVER), avahi.DBUS_INTERFACE_SERVER)
@@ -146,13 +237,48 @@ class DaapAvahiInterface(gobject.GObject): #derived from python-daap/examples
         self.browser.connect_to_signal('ItemNew', self.new_service)
         self.browser.connect_to_signal('ItemRemove', self.remove_service)
 
+class DaapHistory(common.LimitedCache):
+    def __init__(self, limit=5, location=None, menu=None, callback=None):
+        common.LimitedCache.__init__(self, limit)
+        
+        if location is None:
+            location = os.path.join(xdg.get_cache_dir(), 'daaphistory.dat')
+        self.location = location
+        self.menu = menu
+        self.callback = callback
+        
+        self.load()
+        
+    def __setitem__(self, item, value):
+        common.LimitedCache.__setitem__(self, item, value)
+        
+        # add new menu item
+        if self.menu is not None and self.callback is not None:
+            menu_item = _smi('hist'+item, ['sep'], item,
+                        callback=lambda *x: self.callback(None, value+(None,)))
+            self.menu.add_item(menu_item)
+        
+    def load(self):
+        with open(self.location, 'rb') as f:
+            try:
+                d = pickle.load(f)
+                self.update(d)
+            except (IOError, EOFError):
+                # no file
+                pass
+        
+    def save(self):
+        with open(self.location, 'wb') as f:
+            pickle.dump(self.cache, f, common.PICKLE_PROTOCOL)
+
+    
 
 class DaapManager:
     '''
         DaapManager is a class that manages DaapConnections, both manual
     and avahi-generated.
     '''
-    def __init__(self, exaile, menu, avahi):
+    def __init__(self, exaile, _menu, avahi):
         '''
             Init!  Create manual menu item, and connect to avahi signal.
         '''
@@ -160,31 +286,46 @@ class DaapManager:
         self.avahi = avahi
         self.panels = {}
 
-        menu_item = gtk.MenuItem(_('Manually...'))
-        menu_item.connect('activate', self.manual_connect)
-        menu.append_item(menu_item)
+        hmenu = menu.Menu(None)
 
+        def hmfactory(menu, parent, context):
+            item = gtk.MenuItem(_('History'))
+            item.set_submenu(hmenu)
+            sens = settings.get_option('plugin/daapclient/history', True)
+            item.set_sensitive(sens)
+            return item
+    
+        _menu.add_item(_smi('manual', [], _('Manually...'),
+            callback=self.manual_connect))
+        _menu.add_item(menu.MenuItem('history', hmfactory, ['manual']))
+        _menu.add_item(_sep('sep', ['history']))
 
         if avahi is not None:
             avahi.connect("connect", self.connect_share)
+            
+        self.history = DaapHistory(5, menu=hmenu, callback=self.connect_share)
 
-    def connect_share(self, obj, (name, addr, port)):
+    def connect_share(self, obj, (name, address, port, svc)):
         '''
             This function is called when a user wants to connec to
         a DAAP share.  It creates a new panel for the share, and
         requests a track list.
         '''
-
-        conn = DaapConnection(name, addr, port)
-
+        conn = DaapConnection(name, address, port)
+        
         conn.connect()
         library = DaapLibrary(conn)
         panel = NetworkPanel(self.exaile.gui.main.window, library, self)
     #    cst = CollectionScanThread(None, panel.net_collection, panel)
     #    cst.start()
         panel.refresh()             # threaded
-        xlgui.get_controller().add_panel(*panel.get_panel())
+        providers.register('main-panel', panel)
         self.panels[name] = panel
+        
+        # history
+        if settings.get_option('plugin/daapclient/history', True):
+            self.history[name] = (name, address, port)
+            self.history.save()
 
     def disconnect_share(self, name):
         '''
@@ -196,10 +337,10 @@ class DaapManager:
     #    panel.library.daap_share.disconnect()
         panel.daap_share.disconnect()
     #    panel.net_collection.remove_library(panel.library)
-        xlgui.get_controller().remove_panel(panel.get_panel()[0])
+        providers.unregister('main-panel', panel)
         del self.panels[name]
 
-    def manual_connect(self, widget):
+    def manual_connect(self, *args):
         '''
             This function is called when the user selects the manual
         connection option from the menu.  It requests a host/ip to connect
@@ -211,17 +352,33 @@ class DaapManager:
         resp = dialog.run()
 
         if resp == gtk.RESPONSE_OK:
-            loc = dialog.get_value()
-            address = str(loc.split(':')[0])
-            try:
-                port = str(loc.split(':')[1])
-            except IndexError:
+            loc = dialog.get_value().strip()
+            host = loc
+            
+            # the port will be anything after the last :
+            p = host.rfind(":")
+
+            # ipv6 literals should have a closing brace before the port
+            b = host.rfind("]")
+            
+            if p > b:
+                try:
+                    port = int(host[p+1:])
+                    host = host[:p]
+                except ValueError:
+                    logger.error('non-numeric port specified')
+                    return
+            else:
                 port = 3689     # if no port specified, use default DAAP port
 
-            nstr = 'custom%s%s' % (address, port)
-    #        print nstr
-            conn = DaapConnection(loc, address, port)
-            self.connect_share(None, (loc, address, port))
+            # if it's an ipv6 host with brackets, strip them
+            if host and host[0] == '[' and host[-1] == ']':
+                host = host[1:-1]
+
+            nstr = 'custom%s%s' % (host, port)
+
+            conn = DaapConnection(loc, host, port)
+            self.connect_share(None, (loc, host, port, None))
 
     def refresh_share(self, name):
         panel = self.panels[name]
@@ -229,7 +386,8 @@ class DaapManager:
         
         # check for changes
         panel.daap_share.session.update()
-        logger.debug('DAAP Server %s returned revision %d ( old: %d ) after update request'
+        logger.debug('DAAP Server %s returned revision %d ( old: %d ) after'
+                    +' update request'
                 % (name, panel.daap_share.session.revision, rev))
         
         # if changes, refresh
@@ -250,7 +408,7 @@ class DaapManager:
             # there's no point in doing this if we're just shutting down, only on
             # disable
             if remove:
-                xlgui.get_controller().remove_panel(panel.get_panel()[0])
+                providers.unregister('main-panel', panel)
 
 
 class DaapConnection(object):
@@ -258,6 +416,10 @@ class DaapConnection(object):
         A connection to a DAAP share.
     """
     def __init__(self, name, server, port):
+        # if it's an ipv6 address
+        if ':' in server and server[0] != '[':
+            server = '['+server+']'
+    
         self.all = []
         self.session = None
         self.connected = False
@@ -282,12 +444,13 @@ class DaapConnection(object):
             self.connected = True
 #        except DAAPError:
         except Exception, inst:
-            #print 's:%s, p:%s (%s, %s)' % (self.server, self.port, type(self.server), type(self.port))
-            logger.warn('Exception: %s' % inst)
+            logger.warning('failed to connect to ({0},{1})'.format(
+                self.server, self.port))
+            logger.debug(traceback.format_exc())
+            
             self.auth = True
             self.connected = False
-#            raise DAAPError
-            raise Exception, inst
+            raise
 
     def disconnect(self):
         """
@@ -311,10 +474,11 @@ class DaapConnection(object):
         self.database = None
         self.all = []
         self.get_database()
-        #print 'conversion'
-#        t = time.time()
+
+        t = time.time()
         self.convert_list()
-        #print '%f, %d tracks' % (time.time()-t, len(self.all))
+        logger.debug('{0} tracks loaded in {1}s'.format(len(self.all),
+                                                        time.time()-t))
 
 
     def get_database(self):
@@ -364,12 +528,13 @@ class DaapConnection(object):
                     except:
                         if field is 'tracknumber':
                             temp.set_tag_raw('tracknumber', [0], notify_changed=False)
-#                        traceback.print_exc(file=sys.stdout)
+                        logger.debug(traceback.format_exc())
 
 
                 #TODO: convert year (asyr) here as well, what's the formula?
                 try:
-                    temp.set_tag_raw("__length", tr.atom.getAtom('astm') / 1000, notify_changed=False)
+                    temp.set_tag_raw("__length", tr.atom.getAtom('astm') / 1000,
+                                                         notify_changed=False)
                 except:
                     temp.set_tag_raw("__length", 0, notify_changed=False)
 
@@ -427,7 +592,8 @@ class DaapLibrary(collection.Library):
             count = 0
 
         if count > 0:
-            logger.info('Adding %d tracks from %s. (%f s)' % (count, self.daap_share.name, time.time()-t))
+            logger.info('Adding %d tracks from %s. (%f s)' % (count, 
+                                    self.daap_share.name, time.time()-t))
             self.collection.add_tracks(self.daap_share.all)
 
 
@@ -460,21 +626,29 @@ class NetworkPanel(CollectionPanel):
         self.daap_share = library.daap_share
         self.net_collection = collection.Collection(self.name)
         self.net_collection.add_library(library)
-        CollectionPanel.__init__(self, parent, self.net_collection, self.name, _show_collection_empty_message=False)
+        CollectionPanel.__init__(self, parent, self.net_collection, 
+                            self.name, _show_collection_empty_message=False)
 
         self.all = []
+        self.label = self.name
 
         self.connect_id = None
 
-        # Remove the local collection specific menu entries
-        kids = self.menu.get_children()
-        self.menu.remove(kids[-1])
-        self.menu.remove(kids[-2])
-        self.menu.remove(kids[-3])
-
-        # Throw a menu entry on the context menu that can disconnect the DAAP share
-        self.menu.append(_("Refresh Server List"), lambda x, y: mgr.refresh_share(self.name))
-        self.menu.append(_("Disconnect from Server"), lambda x, y: mgr.disconnect_share(self.name))
+        self.menu = menu.Menu(self)
+        def get_tracks_func(*args):
+            return self.tree.get_selected_tracks()
+        self.menu.add_item(menuitems.AppendMenuItem('append', [], 
+                                                            get_tracks_func))
+        self.menu.add_item(menuitems.EnqueueMenuItem('enqueue', ['append'],
+                                                            get_tracks_func))
+        self.menu.add_item(menuitems.PropertiesMenuItem('props', ['enqueue'], 
+                                                            get_tracks_func))
+        self.menu.add_item(_sep('sep',['props']))
+        self.menu.add_item(_smi('refresh', ['sep'], _('Refresh Server List'),
+            callback = lambda *x: mgr.refresh_share(self.name)))
+        self.menu.add_item(_smi('disconnect', ['refresh'], 
+                    _('Disconnect from Server'),
+                    callback = lambda *x: mgr.disconnect_share(self.name)))
 
     @common.threaded
     def refresh(self):
@@ -530,38 +704,33 @@ def __enb(eventname, exaile, wat):
     gobject.idle_add(_enable, exaile)
 
 def _enable(exaile):
-    global MENU_ITEM, MANAGER
+    global MANAGER
 
-#    if not DAAP:
-#        raise Exception("DAAP could not be imported.")
+    event.add_callback(on_settings_change, 'plugin_daapclient_option_set')
 
-#    if not AVAHI:
-#        raise Exception("AVAHI could not be imported.")
+    menu_ = menu.Menu(None)
 
-
-    menu = guiutil.Menu()
-
-    tools = exaile.gui.builder.get_object('tools_menu')
-    MENU_ITEM = gtk.MenuItem(_('Connect to DAAP...'))
-    MENU_ITEM.set_submenu(menu)
-    tools.append(MENU_ITEM)
-    MENU_ITEM.show_all()
+    providers.register('menubar-tools-menu', _sep('plugin-sep', ['track-properties']))
+    
+    item = _smi('daap', ['plugin-sep'], _('Connect to DAAP...'),
+        submenu=menu_)
+    providers.register('menubar-tools-menu', item)
 
     if AVAHI:
         try:
-            avahi_interface = DaapAvahiInterface(exaile, menu)
+            avahi_interface = DaapAvahiInterface(exaile, menu_)
         except RuntimeError: # no dbus?
             avahi_interface = None
-            logger.warn('AVAHI interface could not be initialized (no dbus?)')
+            logger.warning('avahi interface could not be initialized (no dbus?)')
         except dbus.exceptions.DBusException, s:
             avahi_interface = None
             logger.error('Got DBUS error: %s' % s)
-            logger.error('Is avahi-daemon running?')
+            logger.error('is avahi-daemon running?')
     else:
         avahi_interface = None
         logger.warn('AVAHI could not be imported, you will not see broadcast shares.')
 
-    MANAGER = DaapManager(exaile, menu, avahi_interface)
+    MANAGER = DaapManager(exaile, menu_, avahi_interface)
 
 
 
@@ -577,20 +746,29 @@ def disable(exaile):
     '''
         Plugin Disabled.
     '''
-    global MENU_ITEM
     # disconnect from active shares
     if MANAGER is not None:
 #        MANAGER.clear()
         MANAGER.close(True)
 
-
-    if MENU_ITEM:
-        MENU_ITEM.hide()
-        MENU_ITEM.destroy()
-        MENU_ITEM = None
-
+    for item in providers.get('menubar-tools-menu'):
+        if item.name == 'daap':
+            providers.unregister('menubar-tools-menu', item)
+            break
 
     event.remove_callback(__enb, 'gui_loaded')
 
 
+
+# settings stuff
+import daapclientprefs
+
+def get_preferences_pane():
+    return daapclientprefs
+    
+def on_settings_change(event, setting, option):
+    if option == 'plugin/daapclient/ipv6' and MANAGER is not None:
+        MANAGER.avahi.rebuild_share_menu_items()
+
 # vi: et ts=4 sts=4 sw=4
+
