@@ -12,14 +12,24 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import glib, gtk, os
+import glib
+import gtk
+import os
 from egg.trayicon import TrayIcon as EggTrayIcon
-from xl import event, playlist, providers, settings
+
+from xl import (
+    event,
+    player,
+    providers,
+    settings
+)
 from xl.nls import gettext as _
 from xl.trax import Track, is_valid_track
+import xlgui
 from xlgui.guiutil import get_workarea_size
+from xlgui.widgets.playlist import PlaylistPage
 from xlgui.tray import BaseTrayIcon
 
 DROPTRAYICON = None
@@ -123,6 +133,9 @@ class DropTrayIcon(EggTrayIcon, BaseTrayIcon):
             Connects various events
             with callbacks
         """
+        self.connect('size-allocate',
+            self.on_size_allocate)
+
         self.connect('drag-motion',
             self.on_drag_motion)
         self.connect('drag-leave',
@@ -234,6 +247,33 @@ class DropTrayIcon(EggTrayIcon, BaseTrayIcon):
 
         self.drop_target_window.move(x, y)
 
+    def on_size_allocate(self, widget, allocation):
+        """
+            Takes care of resizing the icon if necessary
+        """
+        screen = widget.get_screen()
+        settings = gtk.settings_get_for_screen(screen)
+        icon_sizes = (
+            gtk.ICON_SIZE_MENU, # 1
+            gtk.ICON_SIZE_SMALL_TOOLBAR, # 2
+            gtk.ICON_SIZE_LARGE_TOOLBAR, # 3
+            gtk.ICON_SIZE_BUTTON, # 4
+            gtk.ICON_SIZE_DND, # 5
+            gtk.ICON_SIZE_DIALOG # 6
+        )
+        # Retrieve pixel dimensions of stock icon sizes
+        sizes = [(gtk.icon_size_lookup_for_settings(settings, i)[0], i)
+                 for i in icon_sizes]
+        # Only look at sizes lower than the current dimensions
+        sizes = [(s, i) for s, i in sizes if s <= allocation.width]
+        # Get the closest fit
+        target_size = max(sizes)[1]
+
+        # Avoid setting the same size over and over again
+        if self.image.props.icon_size is not target_size.real:
+            glib.idle_add(self.image.set_from_icon_name,
+                self.image.props.icon_name, target_size)
+
     def on_drag_motion(self, widget, context, x, y, timestamp):
         """
             Prepares to show the drop target
@@ -287,7 +327,7 @@ class DropTrayIcon(EggTrayIcon, BaseTrayIcon):
         """
         self.drop_target_window.hide()
 
-    def on_drag_data_received(self, widget, context, x, y, selection, info, timestamp):
+    def on_drag_data_received(self, widget, context, x, y, selection, info, time):
         """
             Handles dropped data
         """
@@ -304,25 +344,31 @@ class DropTrayIcon(EggTrayIcon, BaseTrayIcon):
         self.drop_target_window.hide()
 
         uris = selection.get_uris()
-        playlist = self.exaile.gui.main.get_selected_playlist()
+        page = xlgui.main.get_selected_page()
 
         if widget is self.play_target:
-            event.add_callback(self.on_tracks_added, 'tracks_added')
+            event.add_callback(self.on_playlist_tracks_added,
+                'playlist_tracks_added')
 
         if widget is self.new_playlist_target:
-            playlist = self.exaile.gui.main.add_playlist()
+            playlist_notebook = xlgui.main.get_playlist_notebook()
+            page = playlist_notebook.create_new_playlist().page
 
-        playlist.drag_data_received(None, context,
-            x, y, selection, info, timestamp)
+        if not isinstance(page, PlaylistPage):
+            return
 
-    def on_tracks_added(self, event_name, playlist, tracks):
+        page.view.emit('drag-data-received',
+            context, x, y, selection, info, time)
+
+    def on_playlist_tracks_added(self, event_name, playlist, tracks):
         """
             Starts playing the newly added tracks
         """
-        if tracks[0] is not None:
-            index = playlist.index(tracks[0])
-            playlist.set_current_pos(index)
-            self.exaile.queue.play(tracks[0])
+        if tracks:
+            position, track = tracks[0]
+            playlist.current_position = position
+            player.QUEUE.play(track)
 
-        event.remove_callback(self.on_tracks_added, 'tracks_added')
+        event.remove_callback(self.on_playlist_tracks_added,
+            'playlist_tracks_added')
 
