@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Adam Olsen
+# Copyright (C) 2012 Brian Parma
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,9 +12,9 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-from xlgui.prefs import widgets
+from xlgui.preferences import widgets
 from xl import xdg
 from xl.nls import gettext as _
 import os
@@ -22,11 +22,28 @@ import os
 name = _('IPython Console')
 basedir = os.path.dirname(os.path.realpath(__file__))
 ui = os.path.join(basedir, 'ipconsole_prefs.ui')
+icon = 'utilities-terminal'
 
-class OpacityPreference(widgets.ScalePrefsItem):
+class OpacityPreference(widgets.ScalePreference):
     default = 80.0
     name = 'plugin/ipconsole/opacity'
 
-class FontPreference(widgets.FontButtonPrefsItem):
+class FontPreference(widgets.FontButtonPreference):
     default = 'Monospace 10'
     name = 'plugin/ipconsole/font'
+    
+class TextColor(widgets.ColorButtonPreference):
+    default = 'lavender'
+    name = 'plugin/ipconsole/text_color'
+    
+class BgColor(widgets.ColorButtonPreference):
+    default = 'black'
+    name = 'plugin/ipconsole/background_color'
+    
+class Theme(widgets.ComboPreference):
+    default = 'Linux'
+    name = 'plugin/ipconsole/iptheme'
+
+class AutoStart(widgets.CheckPreference):
+    default = False
+    name = 'plugin/ipconsole/autostart'
