@@ -30,9 +30,9 @@ import jamtree
 import jamapi
 import menu
 import os
-import urllib
 import hashlib
 from xl import (
+    common,
     event,
     settings,
     providers,
@@ -45,6 +45,7 @@ from xlgui import (
     icons,
     panel
 )
+from xlgui.panel import menus
 from xlgui.widgets.common import DragTreeView
 
 JAMENDO_NOTEBOOK_PAGE = None
@@ -58,14 +59,18 @@ def enable(exaile):
 
 def _enable(eventname, exaile, nothing):
     global JAMENDO_NOTEBOOK_PAGE, COVERS_METHOD
+    
+    user_agent = exaile.get_user_agent_string('jamendo')
+    jamapi.set_user_agent(user_agent)
+    
     JAMENDO_NOTEBOOK_PAGE = JamendoPanel(exaile.gui.main.window, exaile)
-    exaile.gui.add_panel(*JAMENDO_NOTEBOOK_PAGE.get_panel())
-    COVERS_METHOD = JamendoCoverSearch()
+    COVERS_METHOD = JamendoCoverSearch(user_agent)
+    providers.register('main-panel', JAMENDO_NOTEBOOK_PAGE)
     providers.register('covers', COVERS_METHOD)
 
 def disable(exaile):
     global JAMENDO_NOTEBOOK_PAGE, COVERS_METHOD
-    exaile.gui.remove_panel(JAMENDO_NOTEBOOK_PAGE._child)
+    providers.unregister('main-panel', JAMENDO_NOTEBOOK_PAGE)
     providers.unregister('covers', COVERS_METHOD)
 
 class JamendoPanel(panel.Panel):
@@ -78,10 +83,9 @@ class JamendoPanel(panel.Panel):
     ui_info = (os.path.dirname(__file__) + "/ui/jamendo_panel.ui", 'JamendoPanelWindow')
 
     def __init__(self, parent, exaile):
-        panel.Panel.__init__(self, parent)
+        panel.Panel.__init__(self, parent, 'jamendo', "Jamendo")
 
         self.parent = parent
-        self.name = "Jamendo"
         self.exaile = exaile
 
         self.STATUS_READY = _("Ready")
@@ -139,11 +143,7 @@ class JamendoPanel(panel.Panel):
         })
 
         #set up the rightclick menu
-        self.menu = menu.JamendoMenu()
-        self.menu.connect('append-items', lambda *e:
-            self.emit('append-items', self.add_to_playlist()))
-        self.menu.connect('download-items', lambda *e:
-            self.emit('download-items', self.download_selected()))
+        self.menu = menu.JamendoMenu(self)
 
         #setup images
         window = gtk.Window()
@@ -397,16 +397,13 @@ class JamendoPanel(panel.Panel):
 
     #dragdrop stuff
     def drag_data_received(self, *e):
-        print('in drag_data_recieved')
         pass
 
     def drag_data_delete(self, *e):
-        print('in drag_data_delete')
         pass
 
     def drag_get_data(self, treeview, context, selection, target_id, etime):
         self.add_to_playlist()
-        pass
 
 #The following is a custom CoverSearchMethod to retrieve covers from Jamendo
 #It is designed to only to get covers for streaming tracks from jamendo
@@ -416,6 +413,9 @@ class JamendoCoverSearch(CoverSearchMethod):
     fixed = True
     fixed_priority = 5  # take precendence, since we know we are 'right'
                         # for matching tracks.
+                        
+    def __init__(self, user_agent):
+        self.user_agent = user_agent
 
     def find_covers(self, track, limit=-1):
         jamendo_url = track.get_loc_for_io()
@@ -433,8 +433,5 @@ class JamendoCoverSearch(CoverSearchMethod):
 
 
     def get_cover_data(self, url):
-        h = urllib.urlopen(url)
-        data = h.read()
-        h.close()
-        return data
+        return common.get_url_contents(url, self.user_agent)
 
