@@ -1,5 +1,5 @@
 import gtk, glib
-from xl import event, common, playlist
+from xl import event, common, playlist, providers
 from xl import trax
 from xl.nls import gettext as _
 from xlgui import panel, main
@@ -26,6 +26,7 @@ except ImportError:
     md5 = md5.new
 
 def enable(exaile):
+    fp.USER_AGENT = exaile.get_user_agent_string('podcasts')
     if exaile.loading:
         event.add_callback(exaile_ready, 'gui_loaded')
     else:
@@ -36,23 +37,20 @@ def exaile_ready(event, exaile, nothing):
 
     if not PODCASTS:
         PODCASTS = PodcastPanel(main.mainwindow().window)
-        controller = xlgui.get_controller()
-        controller.panels['podcasts'] = PODCASTS
-        controller.add_panel(*PODCASTS.get_panel())
+        providers.register('main-panel', PODCASTS)
 
 def disable(exaile):
     global PODCASTS
 
     if PODCASTS:
-        conroller = xlgui.get_controller()
-        conroller.remove_panel(PODCASTS.get_panel()[0])
+        providers.unregister('main-panel', PODCASTS)
         PODCASTS = None
 
 class PodcastPanel(panel.Panel):
     ui_info = (os.path.join(BASEDIR, 'podcasts.ui'), 'PodcastPanelWindow')
 
     def __init__(self, parent):
-        panel.Panel.__init__(self, parent, _('Podcasts'))
+        panel.Panel.__init__(self, parent, 'podcasts', _('Podcasts'))
         self.podcasts = []
         self.podcast_playlists = playlist.PlaylistManager(
             'podcast_plugin_playlists')
@@ -164,13 +162,14 @@ class PodcastPanel(panel.Panel):
 
             tracks = []
             for e in entries:
-                tr = trax.Track(e['enclosures'][0].href)
-                date = e['updated_parsed']
-                tr.set_tag_raw('artist', title)
-                tr.set_tag_raw('title', e['title'])
-                tr.set_tag_raw('date', "%d-%02d-%02d" %
-                        (date.tm_year, date.tm_mon, date.tm_mday))
-                tracks.append(tr)
+                for link in e.get('enclosures', []):
+                    tr = trax.Track(link.href)
+                    date = e['updated_parsed']
+                    tr.set_tag_raw('artist', title)
+                    tr.set_tag_raw('title', '%s: %s' % (e['title'], link.href.split('/')[-1]))
+                    tr.set_tag_raw('date', "%d-%02d-%02d" %
+                            (date.tm_year, date.tm_mon, date.tm_mday))
+                    tracks.append(tr)
 
             pl.extend(tracks)
             self._set_status('')
@@ -191,7 +190,7 @@ class PodcastPanel(panel.Panel):
     def _open_podcast(self, pl, title):
         new_pl = playlist.Playlist(title)
         new_pl.extend(pl)
-        main.mainwindow().playlist_notebook.create_tab_from_playlist(new_pl)
+        main.get_playlist_notebook().create_tab_from_playlist(new_pl)
 
     @common.threaded
     def _load_podcasts(self):
